@@ -19,6 +19,9 @@ extern char *unixgetcurdir __P_((char *, int));
 #endif
 
 extern int mark;
+#if	FD >= 2
+extern int sorttype;
+#endif
 extern int stackdepth;
 #ifndef	_NOTRADLAYOUT
 extern int tradlayout;
@@ -78,6 +81,7 @@ static char *form_tar[] = {
 	"%a %l %u %g %s %m %d %{yt} %*f",	/* pax */
 	"%10a %u/%g %s %m %d %t %y %*f",	/* tar (UXP/DS) */
 	"%9a %u/%g %s %m %d %t %y %*f",		/* traditional */
+	"%a %u %g %s %m %d %t %y %*f",		/* AIX */
 	"%a %u/%g %m %d %t %y %*f",		/* IRIX */
 #endif	/* !MSDOS */
 	NULL
@@ -210,7 +214,9 @@ launchtable launchlist[MAXLAUNCHTABLE] = {
 	{"*.tar.Z",	"zcat %C|tar tvf -",	PM_TAR, 0},
 	{"*.tar.gz",	"gzip -cd %C|tar tvf -",	PM_TAR, 0},
 	{"*.tar.bz2",	"bzip2 -cd %C|tar tvf -",	PM_TAR, 0},
+# ifndef	CYGWIN
 	{"*.taZ",	"zcat %C|tar tvf -",	PM_TAR, 0},
+# endif
 	{"*.taz",	"gzip -cd %C|tar tvf -",	PM_TAR, 0},
 	{"*.tgz",	"gzip -cd %C|tar tvf -",	PM_TAR, 0},
 #endif	/* !MSDOS */
@@ -239,8 +245,10 @@ archivetable archivelist[MAXARCHIVETABLE] = {
 					"gzip -cd %C|tar xf - %TA", 0},
 	{"*.tar.bz2",	"tar cf - %T|bzip2 -c > %C",
 					"bzip2 -cd %C|tar xf - %TA", 0},
+# ifndef	CYGWIN
 	{"*.taZ",	"tar cf - %T|compress -c > %C",
 					"zcat %C|tar xf - %TA", 0},
+# endif
 	{"*.taz",	"tar cf - %T|gzip -c > %C",
 					"gzip -cd %C|tar xf - %TA", 0},
 	{"*.tgz",	"tar cf - %T|gzip -c > %C",
@@ -282,7 +290,7 @@ static char *autoformat[] = {
 	" %f %s %x %x %d-%m-%y %t %a",		/* RAR (l) */
 # endif	/* !MSDOS */
 };
-#define	AUTOFORMATSIZ	((int)(sizeof(autoformat) / sizeof(char *)))
+#define	AUTOFORMATSIZ	arraysize(autoformat)
 #endif	/* FD >= 2 */
 
 
@@ -703,7 +711,7 @@ int skip;
 				break;
 			case 'u':
 # ifndef	NOUID
-				cp = sscanf2(buf, "%-*d%$",
+				cp = sscanf2(buf, "%-<*d%$",
 					sizeof(uid_t), &uid);
 				if (cp) tmp -> st_uid = uid;
 				else if ((up = finduid(0, buf)))
@@ -713,7 +721,7 @@ int skip;
 				break;
 			case 'g':
 # ifndef	NOUID
-				cp = sscanf2(buf, "%-*d%$",
+				cp = sscanf2(buf, "%-<*d%$",
 					sizeof(gid_t), &gid);
 				if (cp) tmp -> st_gid = gid;
 				else if ((gp = findgid(0, buf)))
@@ -1055,10 +1063,10 @@ int max;
 
 # ifndef	NOUID
 	getfield(buf, line, skip, list, F_UID);
-	if (sscanf2(buf, "%-*d%$", sizeof(uid_t), &uid)) tmp -> st_uid = uid;
+	if (sscanf2(buf, "%-<*d%$", sizeof(uid_t), &uid)) tmp -> st_uid = uid;
 	else tmp -> st_uid = ((up = finduid(0, buf))) ? up -> uid : (uid_t)-1;
 	getfield(buf, line, skip, list, F_GID);
-	if (sscanf2(buf, "%-*d%$", sizeof(gid_t), &gid)) tmp -> st_gid = gid;
+	if (sscanf2(buf, "%-<*d%$", sizeof(gid_t), &gid)) tmp -> st_gid = gid;
 	else tmp -> st_gid = ((gp = findgid(0, buf))) ? gp -> gid : (gid_t)-1;
 # endif
 	getfield(buf, line, skip, list, F_SIZE);
@@ -2100,7 +2108,11 @@ int flags;
 	launchp = list;
 	filelist = NULL;
 	maxent = 0;
+#if	FD >= 2
+	if (sorttype < 200) sorton = 0;
+#else
 	sorton = 0;
+#endif
 
 	if (readarchive(archivefile, launchp, flags) < 0) {
 		arcflist = NULL;
@@ -2194,12 +2206,19 @@ static char *NEAR genfullpath(path, file, full, tmpdir)
 char *path, *file, *full, *tmpdir;
 #endif
 {
+#if	MSDOS
+	int drive;
+#endif
 	char *cp;
 
 	cp = file;
 #ifdef	_USEDOSPATH
+# if	MSDOS
+	if ((drive = _dospath(cp))) cp += 2;
+# else
 	if (_dospath(cp)) cp += 2;
-#endif
+# endif
+#endif	/* _USEDOSPATH */
 
 #ifndef	_NODOSDRIVE
 	if (tmpdir) strcpy(path, tmpdir);
@@ -2214,16 +2233,11 @@ char *path, *file, *full, *tmpdir;
 		return(NULL);
 	}
 #if	MSDOS
-	else if (cp > file) {
-		int drive;
-
-		path[0] = *file;
-		path[1] = ':';
-		path[2] = _SC_;
-		if ((drive = toupper2(*file)) == toupper2(*full))
-			strcpy(&(path[3]), &(full[3]));
-		else if (!unixgetcurdir(&(path[3]), drive - 'A' + 1))
-			return(NULL);
+	else if (drive) {
+		file = gendospath(path, drive, _SC_);
+		drive = toupper2(drive);
+		if (drive == toupper2(*full)) strcpy(file, &(full[3]));
+		else if (!unixgetcurdir(file, drive - 'A' + 1)) return(NULL);
 	}
 #endif
 	else strcpy(path, full);
@@ -2352,7 +2366,7 @@ char *arc;
 	if (tmpdest) {
 		if (!ret) {
 			if (_chdir2(tmpdest) < 0) ret = 1;
-			else forcemovefile(dest);
+			else VOID_C forcemovefile(dest);
 		}
 		free(dest);
 		removetmp(tmpdest, NULL);
@@ -2500,7 +2514,7 @@ int tr, flags;
 
 #ifndef	_NODOSDRIVE
 	if (tmpdest) {
-		if (!ret) forcemovefile(dest);
+		if (!ret) VOID_C forcemovefile(dest);
 		free(dest);
 		removetmp(tmpdest, NULL);
 	}
