@@ -14,7 +14,6 @@
 #endif
 
 #if	MSDOS
-extern int getcurdrv __P_((VOID_A));
 extern int setcurdrv __P_((int, int));
 #endif
 
@@ -27,23 +26,23 @@ extern int setcurdrv __P_((int, int));
 #  ifdef	SVR4
 #  include <sys/sysmacros.h>
 #   ifndef	major
-#   define	major(n)	((((unsigned long)(n)) >> 18) & 0x3fff)
+#   define	major(n)	((((u_long)(n)) >> 18) & 0x3fff)
 #   endif
 #   ifndef	minor
-#   define	minor(n)	(((unsigned long)(n)) & 0x3ffff)
+#   define	minor(n)	(((u_long)(n)) & 0x3ffff)
 #   endif
 #  else
 #   ifndef	major
-#   define	major(n)	((((unsigned)(n)) >> 8) & 0xff)
+#   define	major(n)	((((u_int)(n)) >> 8) & 0xff)
 #   endif
 #   ifndef	minor
-#   define	minor(n)	(((unsigned)(n)) & 0xff)
+#   define	minor(n)	(((u_int)(n)) & 0xff)
 #   endif
 #  endif
 # endif
 #endif
 
-#define	CL_NORM		0
+#define	CL_REG		0
 #define	CL_BACK		1
 #define	CL_DIR		2
 #define	CL_RONLY	3
@@ -53,6 +52,7 @@ extern int setcurdrv __P_((int, int));
 #define	CL_FIFO		7
 #define	CL_BLOCK	8
 #define	CL_CHAR		9
+#define	CL_EXE		10
 #define	ANSI_FG		8
 #define	ANSI_BG		9
 
@@ -95,6 +95,7 @@ static int NEAR putfilename __P_((char *, namelist *, int));
 static VOID NEAR infobar __P_((VOID_A));
 static int NEAR calclocate __P_((int));
 static int NEAR calcfilepos __P_((namelist *, int, char *));
+static int NEAR listupmyself __P_((char *));
 #ifndef	_NOSPLITWIN
 static int NEAR listupwin __P_((char *));
 #endif
@@ -129,18 +130,47 @@ char fullpath[MAXPATHLEN] = "";
 char *macrolist[MAXMACROTABLE];
 int maxmacro = 0;
 int isearch = 0;
-char *helpindex[10] = {
-#ifdef	_NOTREE
-	"help", "eXec", "Copy", "Delete", "Rename",
-	"Sort", "Find", "Logdir", "Editor", "Unpack"
-#else
-	"Logdir", "eXec", "Copy", "Delete", "Rename",
-	"Sort", "Find", "Tree", "Editor", "Unpack"
+#if	FD >= 2
+int helplayout = 0;
 #endif
-};
-char typesymlist[] = "dbclsp";
-u_short typelist[] = {
-	S_IFDIR, S_IFBLK, S_IFCHR, S_IFLNK, S_IFSOCK, S_IFIFO
+char *helpindex[MAXHELPINDEX] = {
+#ifdef	_NOTREE
+	"help",
+#else
+	"Logdir",
+#endif
+	"eXec", "Copy", "Delete", "Rename", "Sort", "Find",
+#ifdef	_NOTREE
+	"Logdir",
+#else
+	"Tree",
+#endif
+	"Editor",
+#ifdef	_NOARCHIVE
+	"",
+#else
+	"Unpack",
+#endif
+#if	FD >= 2
+	"Attr", "Info", "Move", "rmDir", "mKdir",
+	"sHell",
+# ifdef	_NOWRITEFS
+	"",
+# else
+	"Write",
+# endif
+# ifdef	_NOARCHIVE
+	"",
+# else
+	"Backup",
+# endif
+	"View",
+# ifdef	_NOARCHIVE
+	"",
+# else
+	"Pack",
+# endif
+#endif	/* FD >= 2 */
 };
 #ifdef	HAVEFLAGS
 char fflagsymlist[] = "ANacuacu";
@@ -160,25 +190,31 @@ char *precedepath = NULL;
 #endif
 #ifndef	_NOSPLITWIN
 int windows = 1;
+#endif
+#if	!defined (_NOSPLITWIN) || !defined (_NOPTY)
 int win = 0;
 #endif
 int calc_x = -1;
 int calc_y = -1;
 
+static CONST char typesymlist[] = "dbclsp";
+static CONST u_short typelist[] = {
+	S_IFDIR, S_IFBLK, S_IFCHR, S_IFLNK, S_IFSOCK, S_IFIFO
+};
 static CONST u_short modelist[] = {
 	S_IFDIR, S_IFLNK, S_IFSOCK, S_IFIFO, S_IFBLK, S_IFCHR
 };
-#define	MAXMODELIST	((int)sizeof(modelist) / sizeof(u_short))
+#define	MAXMODELIST	arraysize(modelist)
 static CONST char suffixlist[] = {
 	'/', '@', '=', '|'
 };
-#define	MAXSUFFIXLIST	((int)sizeof(suffixlist) / sizeof(char))
+#define	MAXSUFFIXLIST	arraysize(suffixlist)
 #ifndef	_NOCOLOR
 static CONST u_char colorlist[] = {
 	CL_DIR, CL_LINK, CL_SOCK, CL_FIFO, CL_BLOCK, CL_CHAR
 };
 static CONST char defpalette[] = {
-	ANSI_FG,	/* CL_NORM */
+	ANSI_FG,	/* CL_REG */
 	ANSI_BG,	/* CL_BACK */
 	ANSI_CYAN,	/* CL_DIR */
 	ANSI_GREEN,	/* CL_RONLY */
@@ -188,8 +224,9 @@ static CONST char defpalette[] = {
 	ANSI_RED,	/* CL_FIFO */
 	ANSI_FG,	/* CL_BLOCK */
 	ANSI_FG,	/* CL_CHAR */
+	ANSI_FG,	/* CL_EXE */
 };
-#endif
+#endif	/* !_NOCOLOR */
 #ifndef	_NOPRECEDE
 static int maxstat = 0;
 static int haste = 0;
@@ -209,8 +246,9 @@ namelist *namep;
 	for (i = 0; i < MAXMODELIST; i++)
 		if ((namep -> st_mode & S_IFMT) == modelist[i])
 			return(colorlist[i]);
+	if (isexec(namep)) return(CL_EXE);
 
-	return(CL_NORM);
+	return(CL_REG);
 }
 
 static int NEAR biascolor(color)
@@ -278,30 +316,48 @@ static VOID NEAR pathbar(VOID_A)
 
 VOID helpbar(VOID_A)
 {
-	int i, j, col, gap, width, len, ofs;
+	int i, j, col, gap, width, len, ofs, max, blk, rest;
+
+#if	FD >= 2
+	if (helplayout) {
+		max = helplayout / 100;
+		blk = helplayout % 100;
+		if (max < 0) max = FUNCLAYOUT / 100;
+		else if (max > MAXHELPINDEX) max = MAXHELPINDEX;
+		if (blk <= 0 || blk > max) blk = max;
+	}
+	else
+#endif
+	{
+		max = FUNCLAYOUT / 100;
+		blk = FUNCLAYOUT % 100;
+	}
 
 	if (ishardomit()) {
-		col = n_column + 1;
+		col = n_column;
 		gap = 0;
 	}
 	else if (iswellomit()) {
-		col = n_column + 1;
+		col = n_column;
 		gap = 1;
 	}
 	else if (isrightomit()) {
-		col = n_column + 1;
+		col = n_column;
 		gap = 2;
 	}
 	else if (isleftshift()) {
-		col = n_column + 1;
-		gap = 3;
-	}
-	else {
 		col = n_column;
 		gap = 3;
 	}
-	width = (col - 4 - gap * 2) / 10 - 1;
-	ofs = (col - (width + 1) * 10 - 2) / 2;
+	else {
+		col = n_column - 1;
+		gap = 3;
+	}
+	if (max > FUNCLAYOUT / 100) gap = 1;
+
+	rest = max - 1 + gap * ((max - 1) / blk);
+	width = (col - 4 - rest) / max;
+	ofs = (n_column - width * max - rest) / 2;
 	if (ofs < 4) ofs = 4;
 
 	Xlocate(0, L_HELP);
@@ -313,9 +369,9 @@ VOID helpbar(VOID_A)
 	Xputch2(isfileflg(dispmode) ? 'F' : ' ');
 #endif
 
-	for (i = 0; i < 10; i++) {
-		Xlocate(ofs + (width + 1) * i + (i / 5) * gap, L_HELP);
-		len = (width - strlen2(helpindex[i])) / 2;
+	for (i = 0; i < max; i++) {
+		Xlocate(ofs + (width + 1) * i + (i / blk) * gap, L_HELP);
+		len = (width - strlen2(helpindex[i]) + 1) / 2;
 		if (len < 0) len = 0;
 		Xputterm(T_STANDOUT);
 		for (j = 0; j < len; j++) Xputch2(' ');
@@ -535,6 +591,18 @@ static VOID NEAR sizebar(VOID_A)
 	Xtflush();
 }
 
+u_int getfmode(c)
+int c;
+{
+	int i;
+
+	c = tolower2(c);
+	for (i = 0; typesymlist[i]; i++)
+		if (c == typesymlist[i]) return(typelist[i]);
+
+	return((u_int)-1);
+}
+
 int putmode(buf, mode, notype)
 char *buf;
 u_int mode;
@@ -544,10 +612,9 @@ int notype;
 
 	i = 0;
 	if (!notype) {
-		for (j = 0; j < (int)sizeof(typelist) / sizeof(u_short); j++)
+		for (j = 0; j < arraysize(typelist); j++)
 			if ((mode & S_IFMT) == typelist[j]) break;
-		buf[i++] = (j < (int)sizeof(typelist) / sizeof(u_short))
-			? typesymlist[j] : '-';
+		buf[i++] = (j < arraysize(typelist)) ? typesymlist[j] : '-';
 	}
 
 	buf[i++] = (mode & S_IRUSR) ? 'r' : '-';
@@ -579,7 +646,7 @@ u_long flags;
 {
 	int i;
 
-	for (i = 0; i < (int)sizeof(fflaglist) / sizeof(u_long); i++)
+	for (i = 0; i < arraysize(fflaglist); i++)
 		buf[i] = (flags & fflaglist[i]) ? fflagsymlist[i] : '-';
 	buf[i] = '\0';
 
@@ -1142,6 +1209,27 @@ int isstandout;
 	return(pos);
 }
 
+static int NEAR listupmyself(def)
+char *def;
+{
+#ifndef	_NOTREE
+	if (treepath) rewritetree();
+	else
+#endif
+#ifndef	_NOCUSTOMIZE
+	if (custno >= 0) rewritecust();
+	else
+#endif
+#ifndef	_NOPTY
+	if (ptylist[win].pid && ptylist[win].status < 0) /*EMPTY*/;
+	else
+#endif
+	if (filelist && (filepos < maxfile || (!filepos && !maxfile)))
+		return(listupfile(filelist, maxfile, def, 1));
+
+	return(-1);
+}
+
 #ifndef	_NOSPLITWIN
 static int NEAR listupwin(def)
 char *def;
@@ -1168,17 +1256,7 @@ char *def;
 	n = -1;
 	for (win = 0; win < windows; win++) {
 		if (win == dupwin) {
-# ifndef	_NOTREE
-			if (treepath) rewritetree();
-			else
-# endif
-# ifndef	_NOCUSTOMIZE
-			if (custno >= 0) rewritecust();
-			else
-# endif
-			if (filelist
-			&& (filepos < maxfile || (!filepos && !maxfile)))
-				n = listupfile(filelist, maxfile, def, 1);
+			n = listupmyself(def);
 			x = win_x;
 			y = win_y;
 		}
@@ -1309,20 +1387,10 @@ int all;
 	pathbar();
 	if (all >= 0) {
 #ifdef	_NOSPLITWIN
-# ifndef	_NOTREE
-		if (treepath) rewritetree();
-		else
-# endif
-# ifndef	_NOCUSTOMIZE
-		if (custno >= 0) rewritecust();
-		else
-# endif
-		if (filelist && filepos < maxfile)
-			listupfile(filelist, maxfile,
-				filelist[filepos].name, 1);
-#else	/* !_NOSPLITWIN */
+		listupmyself(filelist[filepos].name);
+#else
 		listupwin(filelist[filepos].name);
-#endif	/* !_NOSPLITWIN */
+#endif
 	}
 
 	if (!all) {
@@ -1853,6 +1921,9 @@ VOID main_fd(pathlist, evaled)
 char **pathlist;
 int evaled;
 {
+#ifdef	_USEDOSEMU
+	char buf[MAXPATHLEN];
+#endif
 	char *def, *cwd, file[MAXNAMLEN + 1], prev[MAXNAMLEN + 1];
 	int i, argc, ischgdir;
 
@@ -1944,17 +2015,12 @@ int evaled;
 			else copycurpath(file);
 		}
 
-		if (isdotdir(file) != 2) {
-#ifdef	_USEDOSEMU
-			char buf[MAXPATHLEN];
-#endif
-
-			if (chdir3(nodospath(buf, file), 1) < 0) {
-				hideclock = 2;
-				warning(-1, file);
-				strcpy(prev, file);
-				def = prev;
-			}
+		if (isdotdir(file) != 2
+		&& chdir3(nodospath(buf, file), 1) < 0) {
+			hideclock = 2;
+			warning(-1, file);
+			strcpy(prev, file);
+			def = prev;
 		}
 
 #ifdef	_NOARCHIVE
