@@ -20,6 +20,10 @@
 #include "termemu.h"
 #endif
 
+#ifndef	_NOIME
+#include "roman.h"
+#endif
+
 #define	MD5_BUFSIZ	(128 / 32)
 #define	MD5_BLOCKS	16
 
@@ -63,12 +67,16 @@ extern int win_y;
 extern int hideclock;
 #endif
 extern int inruncom;
-#ifndef	_NOKANJICONV
+#ifdef	_USEUNICODE
 extern int unicodebuffer;
 #endif
 #ifndef	_NOPTY
 extern int parentfd;
 #endif
+#ifndef	_NOIME
+extern romantable *romanlist;
+extern int maxromanlist;
+#endif
 
 static VOID NEAR builtinerror __P_((char *[], char *, int));
 #ifdef	_NOORIGSHELL
@@ -77,6 +85,9 @@ static VOID NEAR hitkey __P_((int));
 #define	hitkey(n)
 #endif
 static VOID NEAR fputsmeta __P_((char *, FILE *));
+#if	(!defined (_NOARCHIVE) && !defined(_NOBROWSE)) || !defined (_NOIME)
+static char **NEAR file2argv __P_((FILE *, char *, int));
+#endif
 #ifndef	_NOARCHIVE
 # if	FD >= 2
 static int NEAR getlaunchopt __P_((int, char *[], char *, launchtable *));
@@ -122,6 +133,11 @@ static int NEAR getyesno __P_((int, char *[]));
 #if	!MSDOS && (FD >= 2)
 static int NEAR savetty __P_((int, char *[]));
 #endif
+#ifndef	_NOIME
+static int NEAR setroman __P_((int, char *[]));
+static VOID NEAR disproman __P_((char *, int, FILE *));
+static int NEAR printroman __P_((int, char *[]));
+#endif
 #ifdef	_NOORIGSHELL
 static int NEAR printenv __P_((int, char *[]));
 static int NEAR setalias __P_((int, char *[]));
@@ -174,7 +190,7 @@ static CONST char *builtinerrstr[] = {
 #define	ER_NOTDUMBTERM	13
 	"Cannot execute on dumb term",
 };
-#define	BUILTINERRSIZ	((int)(sizeof(builtinerrstr) / sizeof(char *)))
+#define	BUILTINERRSIZ	arraysize(builtinerrstr)
 
 static CONST builtintable builtinlist[] = {
 #ifndef	_NOARCHIVE
@@ -213,6 +229,10 @@ static CONST builtintable builtinlist[] = {
 #if	!MSDOS && (FD >= 2)
 	{savetty,	BL_SAVETTY},
 #endif
+#ifndef	_NOIME
+	{setroman,	BL_SETROMAN},
+	{printroman,	BL_PRINTROMAN},
+#endif
 #ifdef	_NOORIGSHELL
 # if	FD >= 2
 	{printenv,	BL_SET},
@@ -235,7 +255,7 @@ static CONST builtintable builtinlist[] = {
 # endif
 #endif	/* _NOORIGSHELL */
 };
-#define	BUILTINSIZ	((int)(sizeof(builtinlist) / sizeof(builtintable)))
+#define	BUILTINSIZ	arraysize(builtinlist)
 
 
 static VOID NEAR builtinerror(argv, s, n)
@@ -302,6 +322,87 @@ FILE *fp;
 	}
 }
 
+#if	(!defined (_NOARCHIVE) && !defined(_NOBROWSE)) || !defined (_NOIME)
+static char **NEAR file2argv(fp, s, whole)
+FILE *fp;
+char *s;
+int whole;
+{
+	char *cp, *line, **argv;
+	ALLOC_T size;
+	int i, j, pc, argc, escape, quote, pqoute, quoted;
+
+	argc = 1;
+	argv = (char **)malloc2(2 * sizeof(char *));
+	argv[0] = strdup2(s);
+	argv[1] = NULL;
+	j = escape = 0;
+	quote = pqoute = quoted = '\0';
+	cp = c_realloc(NULL, 0, &size);
+	while ((line = fgets2(fp, 0))) {
+		if (!escape && !quote && *line == '#') {
+			free(line);
+			continue;
+		}
+		escape = 0;
+		for (i = 0; line[i]; i++) {
+			cp = c_realloc(cp, j + 2, &size);
+			pc = parsechar(&(line[i]), -1,
+				'\0', EA_EOLMETA, &quote, &pqoute);
+			if (pc == PC_CLQUOTE) quoted = line[i];
+			else if (pc == PC_WORD) {
+				cp[j++] = line[i++];
+				cp[j++] = line[i];
+			}
+			else if (pc == PC_SQUOTE || pc == PC_DQUOTE)
+				cp[j++] = line[i];
+			else if (pc == PC_ESCAPE) {
+				if (!line[++i]) {
+					escape = 1;
+					break;
+				}
+				cp[j++] = line[i];
+			}
+			else if (pc == PC_OPQUOTE) quoted = '\0';
+			else if (pc != PC_NORMAL) /*EMPTY*/;
+			else if (!strchr(IFS_SET, line[i])) cp[j++] = line[i];
+			else if (j || quoted) {
+				quoted = cp[j] = '\0';
+				argv = (char **)realloc2(argv,
+					(argc + 2) * sizeof(char *));
+				argv[argc++] = strdup2(cp);
+				argv[argc] = NULL;
+				j = 0;
+			}
+		}
+
+		if (escape);
+		else if (quote) cp[j++] = '\n';
+		else if (j || quoted) {
+			quoted = cp[j] = '\0';
+			argv = (char **)realloc2(argv,
+				(argc + 2) * sizeof(char *));
+			argv[argc++] = strdup2(cp);
+			argv[argc] = NULL;
+			j = 0;
+		}
+		free(line);
+
+		if (!whole && !escape && !quote) break;
+	}
+
+	if (j) {
+		cp[j] = '\0';
+		argv = (char **)realloc2(argv, (argc + 2) * sizeof(char *));
+		argv[argc++] = strdup2(cp);
+		argv[argc] = NULL;
+	}
+	free(cp);
+
+	return(argv);
+}
+#endif	/* (!_NOARCHIVE && !_NOBROWSE) || !_NOIME */
+
 #ifndef	_NOARCHIVE
 # if	FD >= 2
 static int NEAR getlaunchopt(n, argv, opts, lp)
@@ -332,21 +433,21 @@ launchtable *lp;
 	if (c == 'f') {
 		i = countvar(lp -> format);
 		lp -> format = (char **)realloc2(lp -> format,
-			sizeof(char *) * (i + 2));
+			(i + 2) * sizeof(char *));
 		lp -> format[i++] = strdup2(cp);
 		lp -> format[i] = NULL;
 	}
 	else if (c == 'i') {
 		i = countvar(lp -> lignore);
 		lp -> lignore = (char **)realloc2(lp -> lignore,
-			sizeof(char *) * (i + 2));
+			(i + 2) * sizeof(char *));
 		lp -> lignore[i++] = strdup2(cp);
 		lp -> lignore[i] = NULL;
 	}
 	else if (c == 'e') {
 		i = countvar(lp -> lerror);
 		lp -> lerror = (char **)realloc2(lp -> lerror,
-			sizeof(char *) * (i + 2));
+			(i + 2) * sizeof(char *));
 		lp -> lerror[i++] = strdup2(cp);
 		lp -> lerror[i] = NULL;
 	}
@@ -487,7 +588,7 @@ launchtable *lp;
 	}
 
 	if (i < argc) {
-		lp -> format = (char **)malloc2(sizeof(char *) * 2);
+		lp -> format = (char **)malloc2(2 * sizeof(char *));
 		lp -> format[0] = strdup2(argv[i]);
 		lp -> format[1] = NULL;
 	}
@@ -949,9 +1050,8 @@ static char **NEAR readargv(sargv, dargv)
 char **sargv, **dargv;
 {
 	FILE *fp;
-	char *cp, *line, **argv;
-	ALLOC_T size;
-	int i, j, n, argc, dargc, meta, quote, quoted;
+	char *cp, **argv;
+	int n, dargc;
 
 	dargc = countvar(dargv);
 	for (n = 1; sargv[n]; n++) {
@@ -976,80 +1076,10 @@ char **sargv, **dargv;
 			return(NULL);
 		}
 
-		argc = 1;
-		argv = (char **)malloc2(2 * sizeof(char *));
-		argv[0] = strdup2(sargv[0]);
-		argv[1] = NULL;
-		j = meta = 0;
-		quote = quoted = '\0';
-		cp = c_realloc(NULL, 0, &size);
-		while ((line = fgets2(fp, 0))) {
-			if (!meta && !quote && *line == '#') {
-				free(line);
-				continue;
-			}
-			meta = 0;
-			for (i = 0; line[i]; i++) {
-				cp = c_realloc(cp, j + 2, &size);
-				if (line[i] == quote) {
-					quoted = quote;
-					quote = '\0';
-				}
-				else if (iskanji1(line, i)) {
-					cp[j++] = line[i++];
-					cp[j++] = line[i];
-				}
-#  ifdef	CODEEUC
-				else if (isekana(line, i)) {
-					cp[j++] = line[i++];
-					cp[j++] = line[i];
-				}
-#  endif
-				else if (quote) cp[j++] = line[i];
-				else if (line[i] == PMETA) {
-					if (!line[++i]) {
-						meta = 1;
-						break;
-					}
-					cp[j++] = line[i];
-				}
-				else if (line[i] == '\'' || line[i] == '"') {
-					quoted = '\0';
-					quote = line[i];
-				}
-				else if (!strchr(IFS_SET, line[i]))
-					cp[j++] = line[i];
-				else if (j || quoted) {
-					quoted = cp[j] = '\0';
-					argv = (char **)realloc2(argv,
-						(argc + 2) * sizeof(char *));
-					argv[argc++] = strdup2(cp);
-					argv[argc] = NULL;
-					j = 0;
-				}
-			}
-			if (meta);
-			else if (quote) cp[j++] = '\n';
-			else if (j || quoted) {
-				quoted = cp[j] = '\0';
-				argv = (char **)realloc2(argv,
-					(argc + 2) * sizeof(char *));
-				argv[argc++] = strdup2(cp);
-				argv[argc] = NULL;
-				j = 0;
-			}
-			free(line);
-		}
+		argv = file2argv(fp, sargv[0], 1);
 		if (fp != stdin) Xfclose(fp);
 		else clearerr(fp);
-		if (j) {
-			cp[j] = '\0';
-			argv = (char **)realloc2(argv,
-				(argc + 2) * sizeof(char *));
-			argv[argc++] = strdup2(cp);
-			argv[argc] = NULL;
-		}
-		free(cp);
+
 		dargv = readargv(argv, dargv);
 		freevar(argv);
 		if (!dargv) return(NULL);
@@ -1263,7 +1293,7 @@ bindtable *bindp;
 	}
 	else if (argc > i) {
 		if (argv[i][0] != BINDCOMMENT
-		|| bindp -> key < K_F(1) || bindp -> key > K_F(10)) {
+		|| bindp -> key < K_F(1) || bindp -> key > K_F(MAXHELPINDEX)) {
 			bindp -> key = ER_SYNTAXERR;
 			bindp -> f_func = i;
 			return(-1);
@@ -1302,7 +1332,7 @@ char *func1, *func2, *cp;
 		return(-2);
 	}
 
-	if (bind.key < K_F(1) || bind.key > K_F(10)) {
+	if (bind.key < K_F(1) || bind.key > K_F(MAXHELPINDEX)) {
 		if (cp) free(cp);
 	}
 	else if (cp) {
@@ -1332,7 +1362,8 @@ int n;
 {
 	freemacro(bindlist[n].f_func);
 	freemacro(bindlist[n].d_func);
-	if (bindlist[n].key >= K_F(1) && bindlist[n].key <= K_F(10)) {
+	if (bindlist[n].key >= K_F(1)
+	&& bindlist[n].key <= K_F(MAXHELPINDEX)) {
 		free(helpindex[bindlist[n].key - K_F(1)]);
 		helpindex[bindlist[n].key - K_F(1)] = strdup2(nullstr);
 	}
@@ -1388,7 +1419,8 @@ bindtable *bindp;
 {
 	int n;
 
-	if (bindp -> key < K_F(1) || bindp -> key > K_F(10)) return(NULL);
+	if (bindp -> key < K_F(1) || bindp -> key > K_F(MAXHELPINDEX))
+		return(NULL);
 	n = bindp -> key - K_F(1);
 #ifndef	_NOCUSTOMIZE
 	if (!strcmp(helpindex[n], orighelpindex[n])) return(NULL);
@@ -1642,7 +1674,7 @@ devinfo *devp;
 	}
 
 # ifdef	HDDMOUNT
-	if (!strncmp(argv[3], "HDD", sizeof("HDD") - 1)) {
+	if (!strncmp(argv[3], "HDD", strsize("HDD"))) {
 		if (argc > 5) {
 			devp -> drive = ER_FEWMANYARG;
 			devp -> head = 0;
@@ -2117,7 +2149,7 @@ char *argv[];
 		skip = 0;
 		for (i = 1; argv[n][i]; i++) {
 			skip = 0;
-			switch(argv[n][i]) {
+			switch (argv[n][i]) {
 				case 'e':
 					if (argv[n][i + 1]) {
 						editor = &(argv[n][i + 1]);
@@ -2502,7 +2534,7 @@ FILE *fp;
 	sum[3] = (u_long)0x10325476;
 
 	n = b = 0;
-	memset(x, 0, sizeof(x));
+	memset((char *)x, 0, sizeof(x));
 	while ((c = Xfgetc(fpin)) != EOF) {
 		if (cl <= (u_long)0xffffffff - (u_long)BITSPERBYTE)
 			cl += (u_long)BITSPERBYTE;
@@ -2518,7 +2550,7 @@ FILE *fp;
 			if (++n >= MD5_BLOCKS) {
 				n = 0;
 				calcmd5(sum, x);
-				memset(x, 0, sizeof(x));
+				memset((char *)x, 0, sizeof(x));
 			}
 		}
 	}
@@ -2527,7 +2559,7 @@ FILE *fp;
 	x[n] |= 1 << (b + BITSPERBYTE - 1);
 	if (n >= 14) {
 		calcmd5(sum, x);
-		memset(x, 0, sizeof(x));
+		memset((char *)x, 0, sizeof(x));
 	}
 	x[14] = cl;
 	x[15] = ch;
@@ -2656,8 +2688,9 @@ char *argv[];
 		return(-1);
 	}
 
-	if (in == M_UTF8 || out == M_UTF8) readunitable(1);
-	else if (in == UTF8 || out == UTF8) readunitable(0);
+#  ifdef	_USEUNICODE
+	if ((i = (in > out) ? in : out) >= UTF8) readunitable(i - UTF8);
+#  endif
 	while ((cp = fgets2(fpin, 0))) {
 		if (in != DEFCODE) {
 			tmp = newkanjiconv(cp, in, DEFCODE, L_OUTPUT);
@@ -2671,7 +2704,9 @@ char *argv[];
 		free(tmp);
 	}
 
+#  ifdef	_USEUNICODE
 	if (!unicodebuffer) discardunitable();
+#  endif
 	if (fpin != stdin) Xfclose(fpin);
 	else clearerr(fpin);
 	if (fpout != stdout) Xfclose(fpout);
@@ -2761,6 +2796,149 @@ char *argv[];
 # endif	/* FD >= 2 */
 #endif	/* !MSDOS */
 
+#ifndef	_NOIME
+static int NEAR setroman(argc, argv)
+int argc;
+char *argv[];
+{
+	FILE *fp;
+	char *file, **args;
+	int i, n, skip, clean;
+
+	file = NULL;
+	skip = clean = 0;
+	for (n = 1; n < argc && argv[n][0] == '-'; n++) {
+		skip = 0;
+		for (i = 1; argv[n][i]; i++) {
+			skip = 0;
+			switch (argv[n][i]) {
+				case 'c':
+					clean = 2;
+					break;
+				case 'r':
+					clean = 1;
+					break;
+				case 'f':
+					if (argv[n][i + 1]) {
+						file = &(argv[n][i + 1]);
+						skip = 1;
+					}
+					else if (n + 1 < argc) {
+						file = argv[++n];
+						skip = 1;
+					}
+					else skip = -1;
+					break;
+				default:
+					skip = -1;
+					break;
+			}
+			if (skip) {
+				skip--;
+				break;
+			}
+		}
+		if (skip) {
+			skip--;
+			break;
+		}
+	}
+	if (skip || (!file && !clean && n >= argc)) {
+		fprintf2(stderr,
+			"%k: usage: %k [-c] [-r] [-f file] [roman [kanji]]]",
+			argv[0], argv[0]);
+		fputnl(stderr);
+		return(-1);
+	}
+
+	if (clean) {
+		freeroman(clean - 1);
+# ifndef	 _NOPTY
+		sendparent(TE_FREEROMAN, clean - 1);
+# endif
+	}
+
+	if (!file) fp = NULL;
+	else if (!(fp = Xfopen(file, "r"))) {
+		builtinerror(argv, file, -1);
+		return(-1);
+	}
+	else for (;;) {
+		args = file2argv(fp, argv[0], 0);
+		i = 1;
+		if (args[i] && !strcommcmp(args[i], BL_SETROMAN)) i++;
+
+		if (!args[i]) /*EMPTY*/;
+		else if (addroman(args[i], args[i + 1]) < 0)
+			builtinerror(argv, args[i], ER_SYNTAXERR);
+# ifndef	 _NOPTY
+		else sendparent(TE_ADDROMAN, args[i], args[i + 1]);
+# endif
+		freevar(args);
+
+		if (Xfeof(fp)) break;
+	}
+	if (fp) Xfclose(fp);
+
+	if (n >= argc) /*EMPTY*/;
+	else if (addroman(argv[n], argv[n + 1]) < 0) {
+		builtinerror(argv, argv[n], ER_SYNTAXERR);
+		return(-1);
+	}
+# ifndef	 _NOPTY
+	else sendparent(TE_ADDROMAN, argv[n], argv[n + 1]);
+# endif
+
+	return(0);
+}
+
+static VOID NEAR disproman(s, n, fp)
+char *s;
+int n;
+FILE *fp;
+{
+	char buf[2 + 1];
+	int i;
+
+	if (s && *s) fprintf2(fp, "%s ", s);
+	fprintf2(fp, "%s \"", romanlist[n].str);
+	for (i = 0; i < R_MAXKANA; i++) {
+		if (!romanlist[n].code[i]) break;
+		VOID_C jis2str(buf, romanlist[n].code[i]);
+		fprintf2(fp, "%k", buf);
+	}
+	fputc('"', fp);
+	fputnl(fp);
+}
+
+static int NEAR printroman(argc, argv)
+int argc;
+char *argv[];
+{
+	int i, n, ret;
+
+	initroman();
+	ret = -1;
+
+	if (argc < 2) {
+		for (i = 0; i < maxromanlist; i++)
+			disproman(BL_SETROMAN, i, stdout);
+		ret = 0;
+	}
+	else for (n = 1; n < argc; n++) {
+		if (!*(argv[n])
+		|| (i = searchroman(argv[n], strlen(argv[n]))) < 0) {
+			builtinerror(argv, argv[n], ER_NOENTRY);
+			continue;
+		}
+		disproman(BL_PRINTROMAN, i, stdout);
+		ret = 0;
+	}
+
+	return(ret);
+}
+#endif	/* !_NOIME */
+
 #ifdef	_NOORIGSHELL
 static int NEAR printenv(argc, argv)
 int argc;
@@ -3159,7 +3337,7 @@ char *argv[];
 		return(0);
 	}
 
-	var = (char **)malloc2(sizeof(char *) * (n + 1));
+	var = (char **)malloc2((n + 1) * sizeof(char *));
 	for (i = 0; i < n; i++) var[i] = list[i];
 	var[i] = NULL;
 	if (addfunction(strdup2(argv[FUNCNAME]), var) < 0) {
@@ -3193,7 +3371,6 @@ char *argv[];
 	setenv2(argv[1], cp, 1);
 	if (cp) free(cp);
 	adjustpath();
-	evalenv();
 
 	return(0);
 }
@@ -3364,7 +3541,6 @@ int flags;
 		i = argc;
 		if ((cp = getenvval(&i, argv)) != (char *)-1 && i == argc) {
 			if (setenv2(argv[0], cp, 0) < 0) error(argv[0]);
-			evalenv();
 			if (cp) free(cp);
 			n = RET_SUCCESS;
 		}
@@ -3414,7 +3590,7 @@ char ***argvp;
 
 	return(argc);
 }
-#endif
+#endif	/* !_NOCOMPLETE */
 
 #ifdef	DEBUG
 VOID freedefine(VOID_A)
@@ -3443,6 +3619,6 @@ VOID freedefine(VOID_A)
 # ifdef	_USEDOSEMU
 	for (i = 0; fdtype[i].name; i++) free(fdtype[i].name);
 # endif
-	for (i = 0; i < 10; i++) free(helpindex[i]);
+	for (i = 0; i < MAXHELPINDEX; i++) free(helpindex[i]);
 }
 #endif
