@@ -8,13 +8,10 @@
 #ifdef	FD
 #include "fd.h"
 #else	/* !FD */
+#include "machine.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include "machine.h"
-#include "printf.h"
-#include "kctype.h"
-#include "pathname.h"
 
 #ifndef	NOUNISTDH
 #include <unistd.h>
@@ -23,6 +20,10 @@
 #ifndef	NOSTDLIBH
 #include <stdlib.h>
 #endif
+
+#include "printf.h"
+#include "kctype.h"
+#include "pathname.h"
 #endif	/* !FD */
 
 #ifdef	USETIMEH
@@ -427,10 +428,10 @@ static CONST char *doserrstr[] = {
 #define	ER_INVALIDSW	7
 	"Invalid switch",
 };
-#define	DOSERRSIZ	((int)(sizeof(doserrstr) / sizeof(char *)))
+#define	DOSERRSIZ	arraysize(doserrstr)
 
-static char dirsort[(sizeof(DIRSORTFLAG) - 1) * 2 + 1] = "";
-static char dirattr[(sizeof(DIRATTRFLAG) - 1) * 2 + 1] = "";
+static char dirsort[strsize(DIRSORTFLAG) * 2 + 1] = "";
+static char dirattr[strsize(DIRATTRFLAG) * 2 + 1] = "";
 static int dirline = 0;
 static char *dirwd = NULL;
 static int dirtype = '\0';
@@ -461,10 +462,7 @@ off_t *totalp, *freep, *bsizep;
 	statfs_t fsbuf;
 
 # if	MSDOS
-	drv[0] = toupper2(path[0]);
-	drv[1] = ':';
-	drv[2] = '\\';
-	drv[3] = '\0';
+	VOID_C gendospath(drv, toupper2(path[0]), _SC_);
 
 	reg.x.ax = 0x71a0;
 	reg.x.bx = 0;
@@ -692,7 +690,7 @@ char *src, *dest;
 	char path[MAXPATHLEN];
 	int len;
 
-	if ((len = Xreadlink(src, path, sizeof(path) - 1)) < 0) return(-1);
+	if ((len = Xreadlink(src, path, strsize(path))) < 0) return(-1);
 	if (Xlstat(dest, &st) >= 0) {
 #  ifndef	NODIRLOOP
 		if (issamebody(src, dest)) return(0);
@@ -848,7 +846,9 @@ CONST VOID_P vp2;
 				else ret = (cp2) ? -1 : 0;
 				break;
 			case 'D':
-				ret = sp1 -> mtim - sp2 -> mtim;
+				if (sp1 -> mtim == sp2 -> mtim) ret = 0;
+				else ret = (sp1 -> mtim > sp2 -> mtim)
+					? 1 : -1;
 				break;
 			case 'G':
 				ret = dir_isdir(sp2) - dir_isdir(sp1);
