@@ -9,6 +9,10 @@
 #include "termio.h"
 #include "func.h"
 
+#ifndef	O_ACCMODE
+#define	O_ACCMODE	(O_RDONLY | O_WRONLY | O_RDWR)
+#endif
+
 #if	!defined (_NODOSDRIVE) && defined (CODEEUC)
 extern int noconv;
 #endif
@@ -16,7 +20,7 @@ extern int noconv;
 #ifdef	_NODOSDRIVE
 # if	defined (DNAMESIZE) && DNAMESIZE < (MAXNAMLEN + 1)
 typedef struct _st_dirent {
-	char buf[sizeof(struct dirent) - DNAMESIZE + MAXNAMLEN + 1];
+	char buf[(int)sizeof(struct dirent) - DNAMESIZE + MAXNAMLEN + 1];
 } st_dirent;
 # else
 typedef struct dirent	st_dirent;
@@ -71,7 +75,7 @@ char *path, *buf;
 
 	drive = *cp;
 	if (!buf) return(drive);
-	len = MAXPATHLEN - 1;
+	len = sizeof(tmp) - 1;
 	if (cp == buf) {
 		strncpy2(tmp, cp, len);
 		cp = tmp;
@@ -124,8 +128,8 @@ char *path;
 	dirpathlist[maxdirpath - 1].path = strdup2(cp);
 #endif
 #ifdef	CYGWIN
-#define	opendir_saw_u_cygdrive	(1 << (8 * (int)sizeof(dirp -> __flags) - 2))
-#define	opendir_saw_s_cygdrive	(1 << (8 * (int)sizeof(dirp -> __flags) - 3))
+#define	opendir_saw_u_cygdrive	(1 << (8 * sizeof(dirp -> __flags) - 2))
+#define	opendir_saw_s_cygdrive	(1 << (8 * sizeof(dirp -> __flags) - 3))
 	if (buf[0] != _SC_ || buf[1])
 		dirp -> __flags |=
 			opendir_saw_u_cygdrive | opendir_saw_s_cygdrive;
@@ -252,14 +256,14 @@ DIR *dirp;
 	if (!dp) return(NULL);
 
 	dest = ((struct dirent *)&buf) -> d_name;
-#ifdef	CYGWIN
+# ifdef	CYGWIN
 	/* Some versions of Cygwin have neither d_fileno nor d_ino */
 	if (dos) {
 		src = ((struct dosdirent *)dp) -> d_name;
-		buf.d_reclen = ((struct dosdirent *)dp) -> d_reclen;
+		wrap_reclen(&buf) = ((struct dosdirent *)dp) -> d_reclen;
 	}
 	else
-#endif
+# endif
 	{
 		src = dp -> d_name;
 		memcpy((char *)(&buf), (char *)dp, dest - (char *)&buf);
@@ -414,7 +418,11 @@ struct stat *stp;
 	char conv[MAXPATHLEN];
 	int n;
 
+#ifdef	_NOROCKRIDGE
+	path = convput(conv, path, 1, 0, NULL, NULL);
+#else
 	path = convput(conv, path, 1, 0, rpath, NULL);
+#endif
 #ifndef	_NODOSDRIVE
 	if (_dospath(path)) n = doslstat(path, stp);
 	else
@@ -471,7 +479,8 @@ int bufsiz;
 
 	path = convput(conv, path, 1, 0, lbuf, &c);
 #ifndef	_NOROCKRIDGE
-	if (*lbuf && (n = rrreadlink(lbuf, lbuf, MAXPATHLEN)) >= 0) /*EMPTY*/;
+	if (*lbuf && (n = rrreadlink(lbuf, lbuf, sizeof(lbuf) - 1)) >= 0)
+		/*EMPTY*/;
 	else
 #endif
 #ifndef	_NODOSDRIVE
@@ -483,7 +492,7 @@ int bufsiz;
 		lbuf[n] = '\0';
 #ifndef	_NOKANJIFCONV
 		path = kanjiconv2(conv, lbuf,
-			MAXPATHLEN - 1, c, DEFCODE, L_FNAME);
+			strsize(conv), c, DEFCODE, L_FNAME);
 #else
 		path = lbuf;
 #endif
@@ -725,6 +734,18 @@ int whence;
 	return(ofs);
 }
 
+int Xftruncate(fd, len)
+int fd;
+off_t len;
+{
+	int n;
+
+	if (fd >= DOSFDOFFSET) n = dosftruncate(fd, len);
+	else n = ftruncate(fd, len);
+
+	return(n);
+}
+
 int Xdup(oldd)
 int oldd;
 {
