@@ -5,6 +5,7 @@
  */
 
 #include "fd.h"
+#include "funcno.h"
 
 #ifndef	_NOPTY
 
@@ -12,6 +13,10 @@
 #include "kanji.h"
 #include "termemu.h"
 
+#ifndef	_NOIME
+#include "roman.h"
+#endif
+
 #ifdef	_NOORIGSHELL
 #include "termio.h"
 #include "wait.h"
@@ -23,25 +28,29 @@ extern int deletealias __P_((char *));
 # endif
 #endif
 
-#ifdef	SIGALRM
-#define	sigalrm()	((!noalrm) ? SIGALRM : 0)
+#ifdef	_NOIME
+#define	MAXPTYMENU	4
 #else
-#define	sigalrm()	0
+#define	MAXPTYMENU	5
 #endif
 
+extern functable funclist[];
 extern int internal_status;
-#ifdef	SIGALRM
-extern int noalrm;
-#endif
 extern int fdmode;
 extern int fdflags;
 extern int wheader;
+extern int lcmdline;
+extern int maxcmdline;
 extern int ptymode;
+extern int ptyinternal;
 extern int ptymenukey;
 extern p_id_t emupid;
 extern int emufd;
 extern int parentfd;
 extern char *ptytmpfile;
+#ifndef	_NOIME
+extern int ime_cont;
+#endif
 
 static int (*lastfunc)__P_((VOID_A)) = NULL;
 static u_long lockflags = 0;
@@ -76,23 +85,25 @@ int options, *statusp;
 static int waitpty(VOID_A)
 {
 	char result[MAXWINDOWS];
-	int i, n, fds[MAXWINDOWS];
+	int i, oldwin, fds[MAXWINDOWS];
 
 	if (lastfunc && (*lastfunc)() < 0) return(-1);
 
-	for (i = 0; i < MAXWINDOWS; i++) fds[i] = ptylist[i].pipe;
-	if (selectpty(-1, fds, result, 0) > 0) {
+	oldwin = win;
+	for (i = 0; i < MAXWINDOWS; i++)
+		fds[i] = (ptylist[i].pid) ? ptylist[i].pipe : -1;
+	if (selectpty(MAXWINDOWS, fds, result, 0) > 0) {
 		for (i = 0; i < MAXWINDOWS; i++)
 			if (result[i] && ptylist[i].pid) recvchild(i);
 	}
 
+	if (win != oldwin || !emupid) {
+		recvchild(oldwin);
+		return(-2);
+	}
 	if (!(ptylist[win].pid)) return(0);
 	if (ptylist[win].status >= 0) return(-1);
-	if (waitstatus(ptylist[win].pid, WNOHANG, &n) < 0) return(0);
-	ptylist[win].pid = (p_id_t)0;
-	ptylist[win].status = n;
-
-	return(-1);
+	return(checkpty(win));
 }
 
 VOID Xttyiomode(isnl)
@@ -240,12 +251,7 @@ va_dcl
 	va_end(args);
 	if (n < 0) error("malloc()");
 
-	if (!emupid) VOID_C cputs2(buf);
-	else {
-		sendword(emufd, TE_CPUTS2);
-		sendword(emufd, n);
-		sendbuf(emufd, buf, n);
-	}
+	Xcputs2(buf);
 	free(buf);
 
 	return(n);
@@ -368,7 +374,7 @@ VOID changekcode(VOID_A)
 
 VOID changeinkcode(VOID_A)
 {
-	ptylist[win].incode = ptyinkcode;
+	ptylist[win].incode = (u_char)ptyinkcode;
 	if (!emupid) return;
 
 	sendword(emufd, TE_CHANGEINKCODE);
@@ -378,7 +384,7 @@ VOID changeinkcode(VOID_A)
 
 VOID changeoutkcode(VOID_A)
 {
-	ptylist[win].outcode = ptyoutkcode;
+	ptylist[win].outcode = (u_char)ptyoutkcode;
 	if (!emupid) return;
 
 	sendword(emufd, TE_CHANGEOUTKCODE);
@@ -389,12 +395,37 @@ VOID changeoutkcode(VOID_A)
 
 static int NEAR ptygetkey(VOID_A)
 {
-	char *cp, *str[4];
-	int n, c, ch, val[4];
+#ifndef	_NOIME
+	static char buf[MAXKANJIBUF + 1] = "";
+	static int next = 0;
+	u_int w;
+#endif
+	char *cp, *str[MAXPTYMENU];
+	int n, c, ch, val[MAXPTYMENU];
+
+#ifndef	_NOIME
+	if (next > 0) {
+		if (next >= sizeof(buf) || !(c = (u_char)(buf[next++])))
+			next = 0;
+		else return(c);
+	}
+	if (ime_cont) {
+		c = ime_inputkanji(sigalrm(1), buf);
+		if (c < 0 || !ime_cont) movepos(filepos, 0);
+		if (c != K_ESC) {
+			if (c < 0) {
+				next = 0;
+				ime_inputkanji(0, NULL);
+			}
+			else if (!c && (c = (u_char)(buf[0]))) next = 1;
+			return(c);
+		}
+	}
+#endif	/* !_NOIME */
 
 	for (;;) {
 		n = -1;
-		c = getkey2(sigalrm());
+		c = getkey2(sigalrm(1));
 		while (lockflags & (1 << win)) {
 			kbhit2(1000000L / SENSEPERSEC);
 			waitpty();
@@ -407,45 +438,97 @@ static int NEAR ptygetkey(VOID_A)
 		str[0] = asprintf3(PTYAI_K, getkeysym(ptymenukey, 0));
 		str[1] = PTYIC_K;
 		str[2] = PTYBR_K;
-		str[3] = PTYNW_K;
+#ifndef	_NOIME
+		str[3] = PTYKJ_K;
+#endif
+		str[MAXPTYMENU - 1] = PTYNW_K;
 		val[0] = 0;
 		val[1] = 1;
 		val[2] = 2;
+#ifndef	_NOIME
 		val[3] = 3;
+#endif
+		val[MAXPTYMENU - 1] = 4;
 
 		n = 0;
 		changewin(MAXWINDOWS, (p_id_t)-1);
-		ch = selectstr(&n, (windows > 1) ? 4 : 3, 0, str, val);
+		ch = selectstr(&n, (windows > 1) ? MAXPTYMENU : MAXPTYMENU - 1,
+			0, str, val);
+		movepos(filepos, 0);
 		changewin(win, (p_id_t)-1);
 		free(str[0]);
-		if (ch != K_CR) /*EMPTY*/;
+
+		if (ch != K_CR) continue;
 		else if (!n) break;
 		else if (n == 2) {
 			killpty(win, NULL);
+			rewritefile(1);
 			c = -1;
 			break;
 		}
+#ifndef	_NOIME
 		else if (n == 3) {
+			c = ime_inputkanji(sigalrm(1), buf);
+			if (c < 0 || !ime_cont) movepos(filepos, 0);
+			if (c != K_ESC) {
+				if (c < 0) {
+					next = 0;
+					ime_inputkanji(0, NULL);
+				}
+				else if (!c && (c = (u_char)(buf[0])))
+					next = 1;
+				break;
+			}
+		}
+#endif	/* !_NOIME */
+#ifndef	_NOSPLITWIN
+		else if (n == 4) {
 			VOID_C nextwin();
 			c = -2;
 			break;
 		}
+#endif
 		else {
 			changewin(MAXWINDOWS, (p_id_t)-1);
+			lcmdline = -1;
+			maxcmdline = 1;
 			cp = inputstr(PTYKC_K, 1, -1, NULL, -1);
+			movepos(filepos, 0);
 			changewin(win, (p_id_t)-1);
-			if (cp) {
-				c = getkeycode(cp, 0);
-				free(cp);
-				if (c >= 0) break;
-				warning(0, VALNG_K);
+			if (!cp) continue;
+#ifdef	_NOIME
+			c = getkeycode(cp, 0);
+			free(cp);
+			if (c >= 0) break;
+#else	/* !_NOIME */
+			w = ime_getkeycode(cp);
+			free(cp);
+			if (!w) /*EMPTY*/;
+			else if (!(w & ~01777)) {
+				c = w;
+				break;
 			}
-		}
+			else if ((c = ime_inkanjiconv(buf, w)) < 0) /*EMPTY*/;
+			else if (c || !(c = (u_char)(buf[0]))) break;
+			else {
+				next = 1;
+				break;
+			}
+#endif	/* !_NOIME */
 
-		rewritefile(1);
+			changewin(MAXWINDOWS, (p_id_t)-1);
+			warning(0, VALNG_K);
+			movepos(filepos, 0);
+			changewin(win, (p_id_t)-1);
+		}
 	}
 
-	if (n >= 0) rewritefile(1);
+#ifndef	_NOIME
+	if (c < 0) {
+		next = 0;
+		ime_inputkanji(0, NULL);
+	}
+#endif
 
 	return(c);
 }
@@ -615,12 +698,12 @@ int w;
 	keyseq_t key;
 	char *cp, *func1, *func2, ***varp, **var;
 	u_char uc;
-	int n, fd, val;
+	int n, fd, val, wastty;
 
 	fd = ptylist[w].pipe;
 	if (recvbuf(fd, &uc, sizeof(uc)) < 0) return;
 
-	switch(uc) {
+	switch (uc) {
 		case TE_SETVAR:
 			if (recvbuf(fd, &varp, sizeof(varp)) < 0
 			|| recvvar(fd, &var) < 0)
@@ -665,7 +748,6 @@ int w;
 			setenv2(cp, func1, n);
 			free(cp);
 			if (func1) free(func1);
-			evalenv();
 			break;
 		case TE_ADDFUNCTION:
 			if (recvvar(fd, &var) < 0) break;
@@ -686,21 +768,18 @@ int w;
 			|| recvstring(fd, &cp) < 0 || !cp)
 				break;
 			if (putexportvar(cp, n) < 0) free(cp);
-			evalenv();
 			break;
 		case TE_PUTSHELLVAR:
 			if (recvbuf(fd, &n, sizeof(n)) < 0
 			|| recvstring(fd, &cp) < 0 || !cp)
 				break;
 			if (putshellvar(cp, n) < 0) free(cp);
-			evalenv();
 			break;
 		case TE_UNSET:
 			if (recvbuf(fd, &n, sizeof(n)) < 0
 			|| recvstring(fd, &cp) < 0 || !cp)
 				break;
 			VOID_C unset(cp, n);
-			evalenv();
 			free(cp);
 			break;
 		case TE_SETEXPORT:
@@ -756,7 +835,7 @@ int w;
 			|| recvbuf(fd, &val, sizeof(val)) < 0
 			|| recvstring(fd, &cp) < 0 || !cp)
 				break;
-			entryhist(n, cp, val);
+			VOID_C entryhist(n, cp, val);
 			free(cp);
 			break;
 		case TE_ADDKEYBIND:
@@ -784,7 +863,7 @@ int w;
 			|| recvbuf(fd, &(key.len), sizeof(key.len)) < 0
 			|| recvstring(fd, &(key.str)) < 0)
 				break;
-			setkeyseq(key.code, key.str, key.len);
+			VOID_C setkeyseq(key.code, key.str, key.len);
 			break;
 #ifndef	_NOARCHIVE
 		case TE_ADDLAUNCH:
@@ -829,11 +908,11 @@ int w;
 			|| recvbuf(fd, &dev, sizeof(dev)) < 0
 			|| recvstring(fd, &(dev.name)) < 0)
 				break;
-			insertdrv(n, &dev);
+			VOID_C insertdrv(n, &dev);
 			break;
 		case TE_DELETEDRV:
 			if (recvbuf(fd, &n, sizeof(n)) < 0) break;
-			deletedrv(n);
+			VOID_C deletedrv(n);
 			break;
 #endif	/* _USEDOSEMU */
 		case TE_LOCKFRONT:
@@ -850,7 +929,8 @@ int w;
 			break;
 		case TE_SAVETTYIO:
 			if (recvbuf(fd, &n, sizeof(n)) < 0
-			|| recvbuf(fd, &val, sizeof(val)) < 0) break;
+			|| recvbuf(fd, &val, sizeof(val)) < 0)
+				break;
 			if (!val) cp = NULL;
 			else {
 				cp = malloc2(val);
@@ -865,6 +945,40 @@ int w;
 			}
 			else if (cp) free(cp);
 			break;
+#ifndef	_NOIME
+		case TE_ADDROMAN:
+			if (recvstring(fd, &cp) < 0 || !cp) break;
+			if (recvstring(fd, &func1) < 0) {
+				free(cp);
+				break;
+			}
+			VOID_C addroman(cp, func1);
+			break;
+		case TE_FREEROMAN:
+			if (recvbuf(fd, &n, sizeof(n)) < 0) break;
+			freeroman(n);
+			break;
+#endif	/* !_NOIME */
+		case TE_INTERNAL:
+			if (recvbuf(fd, &n, sizeof(n)) < 0
+			|| recvstring(fd, &cp) < 0)
+				break;
+			if (n < 0 || n >= FUNCLISTSIZ) {
+				if (cp) free(cp);
+				break;
+			}
+			keywaitfunc = lastfunc;
+			changewin(MAXWINDOWS, (p_id_t)-1);
+			if (!(wastty = isttyiomode)) Xttyiomode(0);
+			ptyinternal++;
+			internal_status = (*funclist[n].func)(cp);
+			ptyinternal--;
+			if (!wastty) Xstdiomode();
+			changewin(win, (p_id_t)-1);
+			rewritefile(0);
+			keywaitfunc = waitpty;
+			if (cp) free(cp);
+			break;
 		case TE_CHANGESTATUS:
 			if (recvbuf(fd, &n, sizeof(n)) < 0) break;
 			ptylist[w].status = n;
@@ -885,6 +999,7 @@ int frontend(VOID_A)
 	changewsize(wheader, windows);
 	changewin(win, ptylist[win].pid);
 
+	internal_status = FNC_FAIL;
 	if (!(wastty = isttyiomode)) Xttyiomode(1);
 	while ((ch = ptygetkey()) >= 0) sendword(emufd, ch);
 	if (!wastty) Xstdiomode();
@@ -900,7 +1015,8 @@ int frontend(VOID_A)
 			recvchild(win);
 			killpty(win, &status);
 		}
-		internal_status = (status < 128) ? status : FNC_FAIL;
+		if (internal_status == FNC_FAIL && status < 128)
+			internal_status = status;
 	}
 
 	sigvecset(n);
