@@ -65,7 +65,7 @@ extern int errno;
 #define	getconstvar(s)		(char *)getenv(s)
 #define	getshellvar(s, l)	(char *)getenv(s)
 #else
-#define	getconstvar(s)		(getshellvar(s, sizeof(s) - 1))
+#define	getconstvar(s)		(getshellvar(s, strsize(s)))
 #endif
 
 #ifdef	USESTRERROR
@@ -184,11 +184,13 @@ extern int Xopen __P_((char *, int, int));
 #define	Xread		read
 #define	Xwrite		write
 #define	Xlseek		lseek
+#define	Xftruncate	ftruncate
 #else
 extern int Xclose __P_((int));
 extern int Xread __P_((int, char *, int));
 extern int Xwrite __P_((int, char *, int));
 extern off_t Xlseek __P_((int, off_t, int));
+extern int Xftruncate __P_((int, off_t));
 #endif
 #ifdef	_NODOSDRIVE
 #define	Xdup		safe_dup
@@ -267,6 +269,8 @@ extern int ptymacro __P_((char *, char *, int));
 #define	ptysystem(c)		ptymacro(c, NULL, F_DOSYSTEM)
 extern VOID killpty __P_((int, int *));
 extern VOID killallpty __P_((VOID_A));
+extern int checkpty __P_((int));
+extern int checkallpty __P_((VOID_A));
 #endif	/* !_NOPTY */
 
 /* frontend.c */
@@ -338,7 +342,11 @@ extern char *strrchr2 __P_((char *, int));
 extern char *strcpy2 __P_((char *, char *));
 extern char *strncpy2 __P_((char *, char *, int));
 extern int strncpy3 __P_((char *, char *, int *, int));
+#ifdef	CODEEUC
 extern int strlen2 __P_((char *));
+#else
+#define	strlen2	strlen
+#endif
 extern int strlen3 __P_((char *));
 extern int atoi2 __P_((char *));
 extern char *asprintf3 __P_((CONST char *, ...));
@@ -378,11 +386,9 @@ extern int cmptree __P_((CONST VOID_P, CONST VOID_P));
 extern struct dirent *searchdir __P_((DIR *, reg_t *, char *));
 extern int underhome __P_((char *));
 extern int preparedir __P_((char *));
-#ifdef	NOFLOCK
-#define	lockfile(f, m)
-#else
-extern int lockfile __P_((int, int));
-#endif
+extern lockbuf_t *lockopen __P_((char *, int, int));
+extern lockbuf_t *lockfopen __P_((char *, char *, int));
+extern VOID lockclose __P_((lockbuf_t *));
 extern int touchfile __P_((char *, struct stat *));
 #ifdef	_NODOSDRIVE
 #define	nodoslstat	Xlstat
@@ -401,6 +407,7 @@ extern int safemvfile __P_((char *, char *, struct stat *, struct stat *));
 extern char *genrandname __P_((char *, int));
 extern int mktmpdir __P_((char *));
 extern int rmtmpdir __P_((char *));
+extern int opentmpfile __P_((char *, int));
 extern int mktmpfile __P_((char *));
 extern int rmtmpfile __P_((char *));
 extern VOID removetmp __P_((char *, char *));
@@ -415,6 +422,11 @@ extern VOID arrangedir __P_((int));
 #endif
 
 /* apply.c */
+#ifndef	_NOEXTRACOPY
+extern VOID showprogress __P_((off_t));
+extern VOID fshowprogress __P_((char *));
+#endif
+extern int prepareremove __P_((int, int));
 extern int rmvfile __P_((char *));
 extern int rmvdir __P_((char *));
 extern int findfile __P_((char *));
@@ -551,6 +563,8 @@ extern VOID freedefine __P_((VOID_A));
 #define	BL_READLINE	"readline"
 #define	BL_YESNO	"yesno"
 #define	BL_SAVETTY	"savetty"
+#define	BL_SETROMAN	"setroman"
+#define	BL_PRINTROMAN	"printroman"
 #define	BL_ALIAS	"alias"
 #define	BL_UALIAS	"unalias"
 #define	BL_FUNCTION	"function"
@@ -578,10 +592,10 @@ extern int execusercomm __P_((char *, char *, int));
 #endif
 extern int entryhist __P_((int, char *, int));
 extern char *removehist __P_((int));
-extern int loadhistory __P_((int, char *));
+extern int loadhistory __P_((int));
 extern VOID convhistory __P_((char *, FILE *));
-extern int savehistory __P_((int, char *));
-extern int parsehist __P_((char *, int *));
+extern int savehistory __P_((int));
+extern int parsehist __P_((char *, int *, int));
 extern char *evalhistory __P_((char *));
 #ifdef	DEBUG
 extern VOID freehistory __P_((int));
@@ -602,17 +616,23 @@ extern int getlang __P_((char *, int));
 extern int sjis2ujis __P_((char *, u_char *, int));
 extern int ujis2sjis __P_((char *, u_char *, int));
 #endif
-#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+#ifdef	_USEUNICODE
 extern VOID readunitable __P_((int));
 extern VOID discardunitable __P_((VOID_A));
 extern u_int unifysjis __P_((u_int, int));
 extern u_int cnvunicode __P_((u_int, int));
-#endif
+extern VOID ucs2normalization __P_((u_short *, int *, int, u_int, int));
+extern u_int ucs2denormalization __P_((u_short *, int *, int));
+#endif	/* _USEUNICODE */
 #ifndef	_NOKANJICONV
+# ifdef	_USEUNICODE
+extern int ucs2toutf8 __P_((char *, int, u_int));
+extern u_int ucs2fromutf8 __P_((u_char *, int *));
+# endif
 extern int kanjiconv __P_((char *, char *, int, int, int, int));
 extern char *kanjiconv2 __P_((char *, char *, int, int, int, int));
 extern char *newkanjiconv __P_((char *, int, int, int));
-#endif
+#endif	/* !_NOKANJICONV */
 #ifndef	_NOKANJIFCONV
 extern int getkcode __P_((char *));
 #endif
@@ -623,16 +643,41 @@ extern char *convget __P_((char *, char *, int));
 extern char *convput __P_((char *, char *, int, int, char *, int *));
 
 /* input.c */
-extern int intrkey __P_((VOID_A));
+extern int intrkey __P_((int));
 extern int Xgetkey __P_((int, int));
 extern int kanjiputs2 __P_((char *, int, int));
+extern VOID cputspace __P_((int));
+extern VOID cputstr __P_((int, char *));
 extern char *inputstr __P_((char *, int, int, char *, int));
 extern int yesno __P_((CONST char *, ...));
 extern VOID warning __P_((int, char *));
 extern int selectstr __P_((int *, int, int, char *[], int []));
 
+/* ime.c */
+#ifndef	_NOIME
+# ifndef	_NOPTY
+extern u_int ime_getkeycode __P_((char *));
+extern int ime_inkanjiconv __P_((char *, u_int));
+# endif
+extern int ime_inputkanji __P_((int, char *));
+# ifdef	DEBUG
+extern VOID ime_freebuf __P_((VOID_A));
+# endif
+#endif	/* !_NOIME */
+
+/* dict.c */
+#ifndef	_NOIME
+extern VOID discarddicttable __P_((VOID_A));
+extern VOID saveuserfreq __P_((u_short *, u_short *));
+extern VOID freekanjilist __P_((u_short **));
+extern u_short **searchdict __P_((u_short *, int));
+#endif
+
 /* info.c */
 extern VOID help __P_((int));
+#ifndef	NOFLOCK
+extern int isnfs __P_((char *));
+#endif
 extern int writablefs __P_((char *));
 extern off_t getblocksize __P_((char *));
 extern off_t calcKB __P_((off_t, off_t));
@@ -672,7 +717,8 @@ extern char *tree __P_((int, int *));
 #endif
 
 /* custom.c */
-extern VOID evalenv __P_((VOID_A));
+extern VOID initenv __P_((VOID_A));
+extern VOID evalenv __P_((char *, int));
 #ifdef	DEBUG
 extern VOID freeenvpath __P_((VOID_A));
 #endif
@@ -704,6 +750,7 @@ extern int nextwin __P_((VOID_A));
 
 /* browse.c */
 extern VOID helpbar __P_((VOID_A));
+extern u_int getfmode __P_((int));
 extern int putmode __P_((char *, u_int, int));
 #ifdef	HAVEFLAGS
 extern int putflags __P_((char *, u_long));
