@@ -14,7 +14,7 @@
 # ifndef	DJGPP
 # include <dos.h>
 #  ifdef	__TURBOC__
-extern unsigned _stklen = 0x5800;
+extern u_int _stklen = 0x5800;
 #  define	harderr_t	void
 #  else
 #  define	harderr_t	int
@@ -32,6 +32,14 @@ extern unsigned _stklen = 0x5800;
 #define	isorgpid()	(mypid == orgpid)
 #endif
 
+#if	!defined (_NOIME) && defined (DEBUG)
+#include "roman.h"
+#endif
+
+#ifndef	DATADIR
+#define	DATADIR		progpath
+#endif
+
 #if	MSDOS
 # ifndef	BSPATHDELIM
 extern char *adjustpname __P_((char *));
@@ -58,8 +66,7 @@ extern int maxarchive;
 #endif
 extern int wheader;
 extern char fullpath[];
-extern char *histfile;
-extern int savehist;
+extern short savehist[];
 extern char *helpindex[];
 extern int subwindow;
 extern int win_x;
@@ -68,9 +75,12 @@ extern int win_y;
 extern int custno;
 #endif
 extern char *deftmpdir;
-#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+#ifdef	_USEUNICODE
 extern char *unitblpath;
 #endif
+#ifndef	_NOIME
+extern char *dicttblpath;
+#endif
 #ifndef	_NOPTY
 extern int ptymode;
 extern int ptyinternal;
@@ -161,6 +171,7 @@ static int NEAR evaloption __P_((char *[]));
 static char *NEAR searchenv __P_((char *, char *[]));
 static VOID NEAR setexecname __P_((char *));
 static VOID NEAR setexecpath __P_((char *, char *[]));
+int main __P_((int, char *[], char *[]));
 
 char *origpath = NULL;
 char *progpath = NULL;
@@ -824,7 +835,7 @@ VOID title(VOID_A)
 #ifndef	_NOCUSTOMIZE
 VOID saveorigenviron(VOID_A)
 {
-	orighelpindex = copystrarray(NULL, helpindex, NULL, 10);
+	orighelpindex = copystrarray(NULL, helpindex, NULL, MAXHELPINDEX);
 	origbindlist = copybind(NULL, bindlist);
 # ifndef	_NOKEYMAP
 	origkeymaplist = copykeyseq(NULL);
@@ -856,7 +867,7 @@ char **argv;
 	for (i = 1; argv && argv[i]; i++) {
 		if (cp > buf) *(cp++) = ' ';
 		tmp = killmeta(argv[i]);
-		len = snprintf2(cp, sizeof(buf) - (cp - buf), "%s", tmp);
+		len = snprintf2(cp, (int)sizeof(buf) - (cp - buf), "%s", tmp);
 		free(tmp);
 		if (len < 0) break;
 		cp += len;
@@ -1067,7 +1078,6 @@ char *argv[];
 		setenv2(tmp, cp, 0);
 		free(tmp);
 	}
-	evalenv();
 
 	return(i);
 }
@@ -1139,7 +1149,13 @@ char *argv, *envp[];
 VOID initfd(argv)
 char **argv;
 {
-	if (interactive) {
+	int i;
+
+#ifndef	_NOORIGSHELL
+	if (!interactive) /*EMPTY*/;
+	else
+#endif
+	{
 #if	!MSDOS
 		if (adjtty) {
 			Xstdiomode();
@@ -1147,7 +1163,7 @@ char **argv;
 			Xttyiomode(0);
 		}
 #endif	/* !MSDOS */
-		loadhistory(0, histfile);
+		for (i = 0; i < 2; i++) loadhistory(i);
 		entryhist(1, origpath, 1);
 	}
 #ifndef	_NOLOGGING
@@ -1160,11 +1176,16 @@ VOID prepareexitfd(status)
 int status;
 {
 	char cwd[MAXPATHLEN];
+	int i;
 
 #ifndef	_NOLOGGING
 	endlog(status);
 #endif
-	if (interactive && savehist > 0) savehistory(0, histfile);
+#ifndef	_NOORIGSHELL
+	if (!interactive) /*EMPTY*/;
+	else
+#endif
+	for (i = 0; i < 2; i++) savehistory(i);
 
 	cwd[0] = '\0';
 	if (origpath && _chdir2(origpath) < 0) {
@@ -1174,9 +1195,12 @@ int status;
 	}
 	free(origpath);
 	free(progname);
-#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+#ifdef	_USEUNICODE
 	free(unitblpath);
 #endif
+#ifndef	_NOIME
+	free(dicttblpath);
+#endif
 #ifndef	_NODOSDRIVE
 	dosallclose();
 #endif
@@ -1190,7 +1214,7 @@ int status;
 # endif
 # ifndef	_NOCUSTOMIZE
 	if (orighelpindex) {
-		freestrarray(orighelpindex, 10);
+		freestrarray(orighelpindex, MAXHELPINDEX);
 		free(orighelpindex);
 	}
 	free(origbindlist);
@@ -1228,9 +1252,14 @@ int status;
 # ifndef	_NOROCKRIDGE
 	detranspath(NULL, NULL);
 # endif
-# if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+# ifdef	_USEUNICODE
 	discardunitable();
 # endif
+# ifndef	_NOIME
+	ime_freebuf();
+	freeroman(0);
+	discarddicttable();
+# endif
 #endif	/* DEBUG */
 	if (*cwd) rawchdir(cwd);
 }
@@ -1332,7 +1361,8 @@ char *argv[], *envp[];
 	}
 #endif	/* !_NOARCHIVE */
 
-	for (i = 0; i < 10; i++) helpindex[i] = strdup2(helpindex[i]);
+	for (i = 0; i < MAXHELPINDEX; i++)
+		helpindex[i] = strdup2(helpindex[i]);
 #ifdef	_USEDOSEMU
 	for (i = 0; fdtype[i].name; i++) {
 		fdtype[i].name = strdup2(fdtype[i].name);
@@ -1349,13 +1379,13 @@ char *argv[], *envp[];
 	environ[i] = NULL;
 #endif
 
+	initenv();
 	setexecpath(argv[0], envp);
-#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
-# ifdef	DATADIR
+#ifdef	_USEUNICODE
 	unitblpath = strdup2(DATADIR);
-# else
-	unitblpath = strdup2(progpath);
-# endif
+#endif
+#ifndef	_NOIME
+	dicttblpath = strdup2(DATADIR);
 #endif
 
 #ifdef	_NOORIGSHELL
@@ -1372,7 +1402,10 @@ char *argv[], *envp[];
 # endif
 		Xexit2(i);
 	}
-	fdmode = interactive = 1;
+#ifndef	_NOORIGSHELL
+	interactive =
+#endif
+	fdmode = 1;
 	setshellvar(envp);
 	prepareterm();
 	argc = initoption(argc, argv, envp);
