@@ -7,13 +7,10 @@
 #ifdef	FD
 #include "fd.h"
 #else	/* !FD */
+#include "machine.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include "machine.h"
-#include "printf.h"
-#include "kctype.h"
-#include "pathname.h"
 
 # ifndef	NOUNISTDH
 # include <unistd.h>
@@ -22,6 +19,10 @@
 # ifndef	NOSTDLIBH
 # include <stdlib.h>
 # endif
+
+#include "printf.h"
+#include "kctype.h"
+#include "pathname.h"
 #endif	/* !FD */
 
 #if	MSDOS
@@ -153,7 +154,7 @@ char *Xgetwd __P_((char *));
 
 static char *NEAR getenvvar __P_((char *, int));
 static int NEAR setvar __P_((char *, char *, int));
-static int NEAR ismeta __P_((char *s, int, int, int, int));
+static int NEAR isescape __P_((char *s, int, int, int, int));
 #ifdef	_NOORIGGLOB
 static char *NEAR cnvregexp __P_((char *, int));
 #else
@@ -203,6 +204,10 @@ static char *NEAR replacebackquote __P_((char *, int *, char *, int));
 			: (type *)realloc2(ptr, b_size(n, type)))
 #define	getconstvar(s)	(getenvvar(s, sizeof(s) - 1))
 
+char nullstr[] = "";
+char rootpath[] = _SS_;
+char curpath[] = ".";
+char parentpath[] = "..";
 char **argvar = NULL;
 #ifndef	_NOUSEHASH
 hashlist **hashtable = NULL;
@@ -288,7 +293,8 @@ ALLOC_T n, *sizep;
 		*sizep = BUFUNIT;
 		return(malloc2(*sizep));
 	}
-	while (n + 1 >= *sizep) *sizep *= 2;
+	n++;
+	while (n >= *sizep) *sizep *= 2;
 
 	return(realloc2(ptr, *sizep));
 }
@@ -527,13 +533,25 @@ char *path;
 #endif	/* !FD */
 
 #if	MSDOS || (defined (FD) && !defined (_NODOSDRIVE))
+char *gendospath(path, drive, c)
+char *path;
+int drive, c;
+{
+	*(path++) = drive;
+	*(path++) = ':';
+	if (c) *(path++) = c;
+	*path = '\0';
+
+	return(path);
+}
+
 char *strdelim(s, d)
 char *s;
 int d;
 {
 	int i;
 
-	if (d && _dospath(s)) return(s + 1);
+	if (d && _dospath(s)) return(&(s[1]));
 	for (i = 0; s[i]; i++) {
 		if (s[i] == _SC_) return(&(s[i]));
 # ifdef	BSPATHDELIM
@@ -551,7 +569,7 @@ int d;
 	char *cp;
 	int i;
 
-	if (d && _dospath(s)) cp = s + 1;
+	if (d && _dospath(s)) cp = &(s[1]);
 	else cp = NULL;
 	for (i = 0; s[i]; i++) {
 		if (s[i] == _SC_) cp = &(s[i]);
@@ -616,6 +634,10 @@ char *s;
 #if	MSDOS || (defined (FD) && !defined (_NODOSDRIVE))
 	if (_dospath(s)) i = 2;
 	else
+#endif
+#ifdef	DOUBLESLASH
+	if (isdslash(s)) i = 1;
+	else
 #endif
 	i = 0;
 	if (!s[i]) return(&(s[i]));
@@ -655,6 +677,13 @@ char *buf, *s1, *s2;
 		i = 2;
 	}
 	else
+#endif
+#ifdef	DOUBLESLASH
+	if (isdslash(s1)) {
+		buf[0] = s1[0];
+		i = 1;
+	}
+	else
 #endif
 	i = 0;
 	if (!s1[i]) cp = &(buf[i]);
@@ -699,13 +728,17 @@ char *buf, *s1, *s2;
 int strcasecmp2(s1, s2)
 char *s1, *s2;
 {
+	int c1, c2;
+
 	for (;;) {
-		if (toupper2(*s1) != toupper2(*s2)) return(*s1 - *s2);
+		c1 = toupper2(*s1);
+		c2 = toupper2(*s2);
+		if (c1 != c2) return(c1 - c2);
 #ifndef	CODEEUC
 		if (issjis1(*s1)) {
 			s1++;
 			s2++;
-			if (*s1 != *s2) return(*s1 - *s2);
+			if (*s1 != *s2) return((u_char)*s1 - (u_char)*s2);
 		}
 #endif
 		if (!*s1) break;
@@ -720,14 +753,18 @@ int strncasecmp2(s1, s2, n)
 char *s1, *s2;
 int n;
 {
+	int c1, c2;
+
 	while (n-- > 0) {
-		if (toupper2(*s1) != toupper2(*s2)) return(*s1 - *s2);
+		c1 = toupper2(*s1);
+		c2 = toupper2(*s2);
+		if (c1 != c2) return(c1 - c2);
 #ifndef	CODEEUC
 		if (issjis1(*s1)) {
 			if (n-- <= 0) break;
 			s1++;
 			s2++;
-			if (*s1 != *s2) return(*s1 - *s2);
+			if (*s1 != *s2) return((u_char)*s1 - (u_char)*s2);
 		}
 #endif
 		if (!*s1) break;
@@ -764,7 +801,7 @@ int len;
 	char *cp;
 
 	if (len < 0) len = strlen(dir);
-	if ((cp = strrdelim2(dir, &(dir[len]))) && !*(cp + 1)) len = cp - dir;
+	if ((cp = strrdelim2(dir, &(dir[len]))) && !cp[1]) len = cp - dir;
 	if (len <= 0 || strnpathcmp(path, dir, len)) return(NULL);
 	if (path[len] && path[len] != _SC_) return(NULL);
 
@@ -849,12 +886,49 @@ char *s;
 	return(s);
 }
 
+int isrootpath(s)
+char *s;
+{
+	return((s[0] == _SC_ && !s[1]) ? 1 : 0);
+}
+
+VOID copyrootpath(s)
+char *s;
+{
+	*(s++) = _SC_;
+	*s = '\0';
+}
+
+VOID copycurpath(s)
+char *s;
+{
+	*(s++) = '.';
+	*s = '\0';
+}
+
+#ifdef	DOUBLESLASH
+int isdslash(s)
+char *s;
+{
+# if	MSDOS || defined (CYGWIN)
+	char *cp;
+
+	if (s[0] != _SC_ || s[1] != _SC_ || !s[2] || s[2] == _SC_) return(0);
+	if ((cp = strdelim(&(s[2]), 0))) return(cp - s);
+
+	return(strlen(s));
+# else
+	return((s[0] == _SC_ && s[1] == _SC_) ? 2 : 0);
+# endif
+}
+#endif	/* DOUBLESLASH */
+
 char *getbasename(s)
 char *s;
 {
 	char *cp;
 
-	if ((cp = strrdelim(s, 1))) return(cp + 1);
+	if ((cp = strrdelim(s, 1))) return(&(cp[1]));
 
 	return(s);
 }
@@ -882,7 +956,7 @@ int *loginp, *restrictedp;
 	return(s);
 }
 
-static int NEAR ismeta(s, ptr, quote, len, flags)
+static int NEAR isescape(s, ptr, quote, len, flags)
 char *s;
 int ptr, quote, len, flags;
 {
@@ -946,7 +1020,7 @@ int len;
 			re[j++] = s[i];
 			continue;
 		}
-		else if (ismeta(s, i, '\0', len, 0)) {
+		else if (isescape(s, i, '\0', len, 0)) {
 			re[j++] = s[i++];
 			re[j++] = s[i];
 		}
@@ -1122,7 +1196,7 @@ int len;
 		metachar = 0;
 		pc = parsechar(&(s[i]), len - i, '\0', 0, &quote, NULL);
 		if (pc == PC_OPQUOTE || pc == PC_CLQUOTE) continue;
-		else if (pc == PC_META) {
+		else if (pc == PC_ESCAPE) {
 			metachar = 1;
 			i++;
 		}
@@ -1379,7 +1453,7 @@ int len;
 static void NEAR duplwild(dst, src)
 wild_t *dst, *src;
 {
-	memcpy(dst, src, sizeof(wild_t));
+	memcpy((char *)dst, (char *)src, sizeof(wild_t));
 	dst -> fixed.s = (char *)malloc2(src -> fixed.size);
 	memcpy(dst -> fixed.s, src -> fixed.s, src -> fixed.len + 1);
 	dst -> path.s = (char *)malloc2(src -> path.size);
@@ -1389,7 +1463,8 @@ wild_t *dst, *src;
 	if (src -> ino) {
 		dst -> ino = (devino_t *)malloc2(src -> nino
 			* sizeof(devino_t));
-		memcpy(dst -> ino, src -> ino, src -> nino * sizeof(devino_t));
+		memcpy((char *)(dst -> ino), (char *)(src -> ino),
+			src -> nino * sizeof(devino_t));
 	}
 #endif	/* !NODIRLOOP */
 }
@@ -1409,6 +1484,9 @@ int argc;
 char ***argvp;
 wild_t *wp;
 {
+#ifdef	DOUBLESLASH
+	int ds;
+#endif
 	DIR *dirp;
 	struct dirent *dp;
 	struct stat st;
@@ -1431,7 +1509,15 @@ wild_t *wp;
 	plen = wp -> path.len;
 	quote = wp -> quote;
 
-	if (wp -> fixed.len) addstrbuf(&(wp -> path), _SS_, 1);
+	if (wp -> fixed.len) addstrbuf(&(wp -> path), rootpath, 1);
+#ifdef	DOUBLESLASH
+	if (wp -> path.len) ds = 0;
+	else if ((ds = isdslash(wp -> s))) {
+		addstrbuf(&(wp -> fixed), wp -> s, ds);
+		addstrbuf(&(wp -> path), wp -> s, ds);
+		wp -> s += ds;
+	}
+#endif
 
 	for (i = w = 0; wp -> s[i] && wp -> s[i] != _SC_; i++) {
 		pc = parsechar(&(wp -> s[i]), -1,
@@ -1446,7 +1532,7 @@ wild_t *wp;
 			addstrbuf(&(wp -> path), &(wp -> s[i]), 1);
 			i++;
 		}
-		else if (pc == PC_META) {
+		else if (pc == PC_ESCAPE) {
 			if (wp -> flags & EA_KEEPMETA)
 				addstrbuf(&(wp -> fixed), &(wp -> s[i]), 1);
 			if (wp -> s[i + 1] == _SC_) continue;
@@ -1473,11 +1559,14 @@ wild_t *wp;
 	if (!(wp -> s[i])) isdir = 0;
 	else {
 		isdir = 1;
-		addstrbuf(&(wp -> fixed), _SS_, 1);
+		addstrbuf(&(wp -> fixed), rootpath, 1);
 	}
 
 	if (!w) {
 		if (wp -> path.len <= plen) w++;
+#ifdef	DOUBLESLASH
+		else if (ds) st.st_mode = S_IFDIR;
+#endif
 		else if (stat2(wp -> path.s, &st) < 0) return(argc);
 
 		wp -> s += i;
@@ -1488,13 +1577,17 @@ wild_t *wp;
 		}
 
 #ifndef	NODIRLOOP
+# ifdef	DOUBLESLASH
+		if (ds) /*EMPTY*/;
+		else
+# endif
 		if (!w) {
 			wp -> ino = (devino_t *)realloc2(wp -> ino,
 				(wp -> nino + 1) * sizeof(devino_t));
 			wp -> ino[wp -> nino].dev = st.st_dev;
 			wp -> ino[(wp -> nino)++].ino = st.st_ino;
 		}
-#endif
+#endif	/* !NODIRLOOP */
 		return(_evalwild(argc, argvp, wp));
 	}
 
@@ -1524,15 +1617,15 @@ wild_t *wp;
 	}
 
 	if (wp -> path.len) cp = wp -> path.s;
-	else if (wp -> fixed.len) cp = _SS_;
-	else cp = ".";
+	else if (wp -> fixed.len) cp = rootpath;
+	else cp = curpath;
 
 	if (!(dirp = Xopendir(cp))) {
 		regexp_free(re);
 		return(argc);
 	}
 	if (wp -> path.len || wp -> fixed.len)
-		addstrbuf(&(wp -> path), _SS_, 1);
+		addstrbuf(&(wp -> path), rootpath, 1);
 
 	while ((dp = Xreaddir(dirp))) {
 		if (isdotdir(dp -> d_name)) continue;
@@ -1570,7 +1663,7 @@ wild_t *wp;
 			dupl.ino[(dupl.nino)++].ino = st.st_ino;
 #endif
 
-			addstrbuf(&(dupl.fixed), _SS_, 1);
+			addstrbuf(&(dupl.fixed), rootpath, 1);
 			argc = _evalwild(argc, argvp, &dupl);
 		}
 		else if (regexp_exec(re, dp -> d_name, 1)) {
@@ -1885,7 +1978,7 @@ char *com, *search;
 			dlen = (next) ? (next++) - cp : strlen(cp);
 			if (!dlen) tmp = NULL;
 			else {
-				tmp = _evalpath(cp, cp + dlen, 0);
+				tmp = _evalpath(cp, &(cp[dlen]), 0);
 				dlen = strlen(tmp);
 			}
 			if (dlen + len + 1 + EXTWIDTH + 1 > size) {
@@ -2120,7 +2213,8 @@ int dlen, exe;
 		if (size + (cp - path) >= MAXPATHLEN) continue;
 		strncpy2(cp, dp -> d_name, size);
 
-		if ((d = isexecute(path, dirok, exe)) < 0) continue;
+		if (isdotdir(dp -> d_name)) d = 1;
+		else if ((d = isexecute(path, dirok, exe)) < 0) continue;
 
 		new = malloc2(size + 1 + 1);
 		strncpy(new, dp -> d_name, size);
@@ -2149,7 +2243,7 @@ char ***argvp;
 	int dlen;
 
 # ifdef	CWDINPATH
-	argc = completefile(file, len, argc, argvp, ".", 1, 2);
+	argc = completefile(file, len, argc, argvp, curpath, 1, 2);
 # endif
 	if (!(next = getconstvar("PATH"))) return(argc);
 	for (cp = next; cp; cp = next) {
@@ -2159,7 +2253,7 @@ char ***argvp;
 # endif
 		next = strchr(cp, PATHDELIM);
 		dlen = (next) ? (next++) - cp : strlen(cp);
-		tmp = _evalpath(cp, cp + dlen, 0);
+		tmp = _evalpath(cp, &(cp[dlen]), 0);
 		dlen = strlen(tmp);
 		argc = completefile(file, len, argc, argvp, tmp, dlen, 2);
 		free(tmp);
@@ -2174,6 +2268,12 @@ int len, argc;
 char ***argvp;
 int exe;
 {
+# if	MSDOS || (defined (FD) && !defined (_NODOSDRIVE))
+	char cwd[4];
+# endif
+# ifdef	DOUBLESLASH
+	int n;
+# endif
 	char *file, *dir;
 	int dlen;
 
@@ -2187,18 +2287,36 @@ int exe;
 # endif
 
 	if ((file = strrdelim(dir, 0))) {
-		dlen += (file == dir) ? 1 : file - dir;
-		return(completefile(file + 1, strlen(file + 1), argc, argvp,
+		if (file == dir) {
+			dlen++;
+			file++;
+		}
+# ifdef	DOUBLESLASH
+		else if ((n = isdslash(path)) && file < &(path[n])) {
+			if (path[n - 1] != _SC_) return(argc);
+			dlen = n;
+			file = &(path[n]);
+			if (*file == _SC_) file++;
+		}
+# endif
+		else dlen += file++ - dir;
+		return(completefile(file, strlen(file), argc, argvp,
 			path, dlen, exe));
 	}
 # ifndef	NOUID
 	else if (*path == '~')
 		return(completeuser(&(path[1]), len - 1, argc, argvp, 1));
 # endif
-	else if (exe && dir == path)
-		return(completeexe(path, len, argc, argvp));
+# if	MSDOS || (defined (FD) && !defined (_NODOSDRIVE))
+	else if (dir != path) {
+		len -= dlen;
+		dlen = gendospath(cwd, *path, '.') - cwd;
+		return(completefile(dir, len, argc, argvp, cwd, dlen, exe));
+	}
+# endif
+	else if (exe) return(completeexe(path, len, argc, argvp));
 
-	return(completefile(path, len, argc, argvp, ".", 1, exe));
+	return(completefile(path, len, argc, argvp, curpath, 1, exe));
 }
 
 char *findcommon(argc, argv)
@@ -2235,7 +2353,7 @@ int quoted;
 	for (i = j = 0; s2[i]; i++, j++) {
 		quote = quoted;
 		pc = parsechar(&(s2[i]), -1, '\0', EA_EOLMETA, &quote, NULL);
-		if (pc == PC_OPQUOTE || pc == PC_CLQUOTE || pc == PC_META) {
+		if (pc == PC_OPQUOTE || pc == PC_CLQUOTE || pc == PC_ESCAPE) {
 			if (s1) s1[j] = PMETA;
 			j++;
 		}
@@ -2260,7 +2378,7 @@ int delim;
 
 	if (!argv) return(NULL);
 	for (i = len = 0; argv[i]; i++) len += strlen(argv[i]);
-	if (i < 1) return(strdup2(""));
+	if (i < 1) return(strdup2(nullstr));
 	len += (delim) ? i - 1 : 0;
 	cp = malloc2(len + 1);
 	len = strcpy2(cp, argv[0]) - cp;
@@ -2320,6 +2438,10 @@ char *s;
 int len, spc, flags, *qp, *pqp;
 {
 	if (*s == *qp) {
+#ifdef	FD
+		if (flags & EA_FINDMETA)
+			return((*qp == '\'') ? PC_EXMETA : PC_META);
+#endif
 		if (!pqp) *qp = '\0';
 		else {
 			*qp = *pqp;
@@ -2332,7 +2454,13 @@ int len, spc, flags, *qp, *pqp;
 	else if (isekana(s, 0)) return(PC_WORD);
 #endif
 	else if (*qp == '\'') return(PC_SQUOTE);
-	else if (ismeta(s, 0, *qp, len, flags)) return(PC_META);
+#ifdef	FD
+	else if ((flags & EA_FINDMETA) && strchr(DQ_METACHAR, *s))
+		return(PC_META);
+	else if ((flags & EA_FINDDELIM) && strchr(CMDLINE_DELIM, *s))
+		return(PC_DELIM);
+#endif
+	else if (isescape(s, 0, *qp, len, flags)) return(PC_ESCAPE);
 	else if (*qp == '`') return(PC_BQUOTE);
 #ifdef	NESTINGQUOTE
 	else if ((flags & EA_BACKQ) && *s == '`') {
@@ -2344,6 +2472,9 @@ int len, spc, flags, *qp, *pqp;
 	else if (spc && *s == spc) return(*s);
 	else if (*qp) return(PC_DQUOTE);
 	else if (!(flags & EA_NOEVALQ) && *s == '\'') {
+#ifdef	FD
+		if (flags & EA_FINDMETA) return(PC_META);
+#endif
 		*qp = *s;
 		return(PC_OPQUOTE);
 	}
@@ -2358,6 +2489,10 @@ int len, spc, flags, *qp, *pqp;
 		return(PC_OPQUOTE);
 	}
 #endif
+#ifdef	FD
+	else if ((flags & EA_FINDMETA) && strchr(METACHAR, *s))
+		return(PC_META);
+#endif
 
 	return(PC_NORMAL);
 }
@@ -2484,7 +2619,7 @@ int qed, nonl, nest;
 #else
 		pc = parsechar(&(s[*ptrp]), -1, '$', EA_BACKQ, &q, NULL);
 #endif
-		if (pc == PC_WORD || pc == PC_META) (*ptrp)++;
+		if (pc == PC_WORD || pc == PC_ESCAPE) (*ptrp)++;
 		else if (pc == '$') {
 			if (!s[++(*ptrp)]) return(0);
 			cp = s;
@@ -2536,13 +2671,13 @@ int plen, mode;
 	ret = NULL;
 	len = strlen(s);
 	if ((mode & ~0x80) != '#') {
-		if (mode & 0x80) for (cp = s; cp < s + len; cp++) {
+		if (mode & 0x80) for (cp = s; cp < &(s[len]); cp++) {
 			if (regexp_exec(re, cp, 0)) {
 				ret = cp;
 				break;
 			}
 		}
-		else for (cp = s + len - 1; cp >= s; cp--) {
+		else for (cp = &(s[len - 1]); cp >= s; cp--) {
 			if (regexp_exec(re, cp, 0)) {
 				ret = cp;
 				break;
@@ -2642,7 +2777,7 @@ int plen, *modep;
 
 			for (i = j = 0; arglist[i + 1]; i++)
 				j += addmeta(NULL, arglist[i + 1], quoted);
-			if (i <= 0) cp = strdup2("");
+			if (i <= 0) cp = strdup2(nullstr);
 			else {
 				j += (i - 1) * 3;
 				cp = malloc2(j + 1);
@@ -3137,7 +3272,7 @@ char **argp;
 	len = ((cp = strdelim(top, 0))) ? (cp - top) : strlen(top);
 	if (!len) cp = gethomedir();
 # ifdef	FD
-	else if (len == (int)sizeof("FD") - 1 && !strnpathcmp(top, "FD", len))
+	else if (len == strsize("FD") && !strnpathcmp(top, "FD", len))
 		cp = progpath;
 # endif
 	else {
@@ -3246,7 +3381,7 @@ int qed, flags;
 			if (*cp == '`') {
 				bbuf[j] = '\0';
 				buf = replacebackquote(buf, &i,
-					bbuf, strlen(cp + 1));
+					bbuf, strlen(&(cp[1])));
 				j = 0;
 			}
 			else if (!(flags & EA_STRIPQ)) buf[i++] = *cp;
@@ -3282,7 +3417,7 @@ int qed, flags;
 				return(NULL);
 			}
 		}
-		else if (pc == PC_META) {
+		else if (pc == PC_ESCAPE) {
 			cp++;
 			if (flags & EA_KEEPMETA) pc = PC_NORMAL;
 			else if (*cp == '$') /*EMPTY*/;
@@ -3297,7 +3432,7 @@ int qed, flags;
 				bbuf[j++] = *cp;
 			}
 			else {
-				if (pc != PC_META) buf[i++] = PMETA;
+				if (pc != PC_ESCAPE) buf[i++] = PMETA;
 				buf[i++] = *cp;
 			}
 		}
@@ -3344,7 +3479,7 @@ char ***argvp, *ifs;
 		for (i = 0, quote = '\0'; (*argvp)[n][i]; i++) {
 			pc = parsechar(&((*argvp)[n][i]), -1,
 				'\0', 0, &quote, NULL);
-			if (pc == PC_WORD || pc == PC_META) i++;
+			if (pc == PC_WORD || pc == PC_ESCAPE) i++;
 			else if (pc != PC_NORMAL) /*EMPTY*/;
 			else if (strchr2(ifs, (*argvp)[n][i])) {
 				for (j = i + 1; (*argvp)[n][j]; j++)
@@ -3429,13 +3564,13 @@ int flags;
 			if (flags & EA_STRIPQ) continue;
 		}
 		else if (pc == PC_WORD) arg[j++] = arg[i++];
-		else if (pc == PC_META) {
+		else if (pc == PC_ESCAPE) {
 			i++;
 			if (flags & EA_KEEPMETA) pc = PC_NORMAL;
 			else if (!quote && !(flags & EA_BACKQ)) /*EMPTY*/;
 			else if (!strchr(DQ_METACHAR, arg[i])) pc = PC_NORMAL;
 
-			if (pc != PC_META) arg[j++] = PMETA;
+			if (pc != PC_ESCAPE) arg[j++] = PMETA;
 			else stripped++;
 		}
 
@@ -3452,7 +3587,10 @@ int flags;
 {
 #if	MSDOS && defined (FD) && !defined (_NOUSELFN)
 	char alias[MAXPATHLEN];
-	int top = -1;
+	int top;
+#endif
+#ifdef	DOUBLESLASH
+	int ds;
 #endif
 	char *cp, *tmp;
 	int i, j, c, pc, size, quote;
@@ -3468,9 +3606,16 @@ int flags;
 	free(cp);
 	cp = tmp;
 
+#if	MSDOS && defined (FD) && !defined (_NOUSELFN)
+	top = -1;
+#endif
+#ifdef	DOUBLESLASH
+	ds = isdslash(cp);
+#endif
 	size = strlen(cp) + 1;
 	tmp = malloc2(size);
 	quote = '\0';
+
 	for (i = j = c = 0; cp[i]; c = cp[i++]) {
 		pc = parsechar(&(cp[i]), -1, '\0', 0, &quote, NULL);
 		if (pc == PC_CLQUOTE) {
@@ -3492,7 +3637,7 @@ int flags;
 			if (!(flags & EA_NOEVALQ)) continue;
 		}
 		else if (pc == PC_WORD) tmp[j++] = cp[i++];
-		else if (pc == PC_META) {
+		else if (pc == PC_ESCAPE) {
 			i++;
 			if ((flags & EA_KEEPMETA)
 			|| (quote && !strchr(DQ_METACHAR, cp[i])))
@@ -3505,9 +3650,12 @@ int flags;
 			if (!(flags & EA_NOEVALQ)) continue;
 		}
 		else if (pc != PC_NORMAL) /*EMPTY*/;
-		else if (!(flags & EA_NOUNIQDELIM)
-		&& cp[i] == _SC_ && c == _SC_)
-			continue;
+		else if (flags & EA_NOUNIQDELIM) /*EMPTY*/;
+#ifdef	DOUBLESLASH
+		else if (ds && i == 1) /*EMPTY*/;
+#endif
+		else if (cp[i] == _SC_ && c == _SC_) continue;
+
 		tmp[j++] = cp[i];
 	}
 	tmp[j] = '\0';
