@@ -9,6 +9,10 @@
 #include <string.h>
 #include <sys/types.h>
 
+#ifndef	NOUNISTDH
+#include <unistd.h>
+#endif
+
 #ifndef	NOSTDLIBH
 #include <stdlib.h>
 #endif
@@ -41,7 +45,17 @@ extern char *newkanjiconv __P_((char *, int, int, int));
 extern int getoutputkcode __P_((VOID_A));
 # endif
 extern char *restorearg __P_((char *));
-#endif
+#endif	/* FD */
+
+#ifdef	CODEEUC
+# ifdef	FD
+extern int strlen2 __P_((char *));
+# else
+static int NEAR strlen2 __P_((char *));
+# endif
+#else	/* !CODEEUC */
+#define	strlen2	strlen
+#endif	/* !CODEEUC */
 
 static int NEAR checkchar __P_((int, printbuf_t *));
 static int NEAR setint __P_((u_long_t, int, printbuf_t *, int, int));
@@ -76,6 +90,18 @@ int printfsize[] = {
 };
 
 
+#if	!defined (FD) && defined (CODEEUC)
+static int NEAR strlen2(s)
+char *s;
+{
+	int i, len;
+
+	for (i = len = 0; s[i]; i++, len++) if (isekana(s, i)) i++;
+
+	return(len);
+}
+#endif	/* !FD && CODEEUC */
+
 int getnum(s, ptrp)
 CONST char *s;
 int *ptrp;
@@ -152,7 +178,7 @@ int width, prec;
 	long_t n;
 	int i, c, len, ptr, sign, bit;
 
-	memcpy(&n, &u, sizeof(n));
+	memcpy((char *)&n, (char *)&u, sizeof(n));
 	bit = 0;
 
 #ifdef	MINIMUMSHELL
@@ -192,7 +218,7 @@ int width, prec;
 	if (!n) {
 		if (prec) num[len++] = '0';
 	}
-	else while (len < (int)sizeof(num) / sizeof(char)) {
+	else while (len < arraysize(num)) {
 #ifdef	MINIMUMSHELL
 		if (!bit) {
 			i = (n % base);
@@ -249,8 +275,10 @@ int width, prec;
 
 	if (prec >= 0 && len < prec) {
 		for (i = 0; i < prec - len; i++) {
+#ifndef	MINIMUMSHELL
 			if ((pbufp -> flags & VF_STRICTWIDTH) && width == 1)
 				break;
+#endif
 			width--;
 			c = '0';
 			if ((pbufp -> flags & VF_THOUSAND)
@@ -259,6 +287,7 @@ int width, prec;
 			if (setchar(c, pbufp) < 0) return(-1);
 		}
 	}
+#ifndef	MINIMUMSHELL
 	if ((pbufp -> flags & VF_STRICTWIDTH) && width >= 0 && width < len)
 	for (i = 0; i < width; i++) {
 		c = '9';
@@ -267,7 +296,9 @@ int width, prec;
 			c = (i) ? ',' : ' ';
 		if (setchar(c, pbufp) < 0) return(-1);
 	}
-	else for (i = 0; i < len; i++) {
+	else
+#endif	/* !MINIMUMSHELL */
+	for (i = 0; i < len; i++) {
 		if (setchar(num[len - i - 1], pbufp) < 0) return(-1);
 	}
 
@@ -285,16 +316,10 @@ int width, prec;
 {
 	int i, j, c, len, total;
 
-	if (s) {
-#ifdef	CODEEUC
-		for (i = len = 0; s[i]; i++, len++) if (isekana(s, i)) i++;
-#else
-		len = strlen(s);
-#endif
-	}
+	if (s) len = strlen2(s);
 	else {
 		s = "(null)";
-		len = (int)sizeof("(null)") - 1;
+		len = strsize("(null)");
 #ifdef	LINUX
 		/* spec. of glibc */
 		if (prec >= 0 && len > prec) len = 0;
@@ -496,29 +521,37 @@ va_list args;
 		}
 
 		if (base) {
-			if (len == (int)sizeof(u_long_t))
-				u = va_arg(args, u_long_t);
-#ifdef	HAVELONGLONG
-			else if (len == (int)sizeof(u_long))
-				u = va_arg(args, u_long);
-#endif
-			else u = va_arg(args, u_int);
-
 #ifndef	HAVELONGLONG
 			if (len > (int)sizeof(u_long_t)) {
-				u_long_t hi, tmp;
-
-				while (len > (int)sizeof(u_int)) {
-					hi = va_arg(args, u_int);
-					len -= (int)sizeof(u_int);
-				}
+				u_long_t tmp;
+				int hi;
 
+# ifndef	HPUX
+	/*
+	 * HP-UX always pushes arguments from lowest to uppermost,
+	 * in spite of the CPU endian.
+	 */
 				tmp = 0x5a;
 				cp = (char *)(&tmp);
 				if (*cp != 0x5a) {
-					tmp = hi;
-					hi = u;
-					u = tmp;
+					hi = va_arg(args, u_int);
+					len -= (int)sizeof(u_int);
+					while (len > (int)sizeof(u_long_t)) {
+						tmp = va_arg(args, u_int);
+						len -= (int)sizeof(u_int);
+					}
+					u = va_arg(args, u_long_t);
+				}
+				else
+# endif	/* !HPUX */
+				{
+					hi = 0;
+					u = va_arg(args, u_long_t);
+					len -= (int)sizeof(u_long_t);
+					while (len > 0) {
+						hi = va_arg(args, u_int);
+						len -= (int)sizeof(u_int);
+					}
 				}
 
 				if (pbufp -> flags & VF_UNSIGNED) {
@@ -526,7 +559,7 @@ va_list args;
 				}
 				else {
 					mask = (MAXUTYPE(u_long_t) >> 1);
-					if (hi & ~mask) {
+					if (hi < 0) {
 						if (++hi || !(u & ~mask))
 							u = ~mask;
 					}
@@ -538,10 +571,20 @@ va_list args;
 			}
 			else
 #endif	/* !HAVELONGLONG */
+			if (len == (int)sizeof(u_long_t))
+				u = va_arg(args, u_long_t);
+#ifdef	HAVELONGLONG
+			else if (len == (int)sizeof(u_long))
+				u = va_arg(args, u_long);
+#endif
+			else u = va_arg(args, u_int);
+
 			if (!(pbufp -> flags & VF_UNSIGNED)) {
-				mask = (MAXUTYPE(u_long_t)
-					>> (((int)sizeof(long_t) - len)
-					* BITSPERBYTE + 1));
+				mask = MAXUTYPE(u_long_t);
+				if (len < (int)sizeof(u_long_t))
+					mask >>= ((int)sizeof(u_long_t) - len)
+						* BITSPERBYTE;
+				mask >>= 1;
 				if (u & ~mask) u |= ~mask;
 			}
 			len = setint(u, base, pbufp, width, prec);
