@@ -9,9 +9,6 @@
 
 #ifndef	_NOROCKRIDGE
 
-extern char typesymlist[];
-extern u_short typelist[];
-
 #define	TRANSTBLFILE	"TRANS.TBL"
 #define	TRANSTBLVAR	1
 #define	RR_TRANS	001
@@ -28,13 +25,13 @@ extern u_short typelist[];
 #  ifdef	SVR4
 #  include <sys/sysmacros.h>
 #   ifndef	makedev
-#   define	makedev(ma, mi)	(((((unsigned long)(ma)) & 0x3fff) << 18) \
-				| (((unsigned long)(mi)) & 0x3ffff))
+#   define	makedev(ma, mi)	(((((u_long)(ma)) & 0x3fff) << 18) \
+				| (((u_long)(mi)) & 0x3ffff))
 #   endif
 #  else
 #   ifndef	makedev
-#   define	makedev(ma, mi)	(((((unsigned)(ma)) & 0xff) << 8) \
-				| (((unsigned)(mi)) & 0xff))
+#   define	makedev(ma, mi)	(((((u_int)(ma)) & 0xff) << 8) \
+				| (((u_int)(mi)) & 0xff))
 #   endif
 #  endif
 # endif
@@ -49,7 +46,7 @@ typedef dev_t	r_dev_t;
 typedef struct _transtable {
 	char *org;
 	char *alias;
-	char *symlink;
+	char *slink;
 	r_dev_t rdev;
 	char type;
 	struct _transtable *next;
@@ -109,7 +106,7 @@ int len, *flagsp;
 
 	len = cp - buf;
 	if (len + 1 >= MAXPATHLEN - 1) return(NULL);
-	snprintf2(&(buf[len]), sizeof(buf) - len, ";%d", TRANSTBLVAR);
+	snprintf2(&(buf[len]), (int)sizeof(buf) - len, ";%d", TRANSTBLVAR);
 	*flagsp |= RR_VERNO;
 	if ((fp = Xfopen(buf, "r"))) return(fp);
 
@@ -171,7 +168,7 @@ int len;
 			continue;
 		}
 
-		for (eol = cp; *eol && *eol != ' ' && *eol != '\t'; eol++);
+		for (eol = cp; *eol && !isblank2(*eol); eol++);
 		if (*eol) *(eol++) = '\0';
 		if (!*(eol = skipspace(eol))) {
 			free(line);
@@ -180,15 +177,14 @@ int len;
 
 		org = getorgname(cp, flags);
 		cp = eol;
-		while (*eol && *eol != ' ' && *eol != '\t') eol++;
+		while (*eol && !isblank2(*eol)) eol++;
 		l1 = eol - cp;
 		if (*eol) *(eol++) = '\0';
 		l2 = 0;
 		maj = min = (r_dev_t)-1;
 		if (*line == 'L') {
 			eol = skipspace(eol);
-			while (eol[l2] && eol[l2] != ' ' && eol[l2] != '\t')
-				l2++;
+			while (eol[l2] && !isblank2(eol[l2])) l2++;
 			eol[l2++] = '\0';
 		}
 		else if (*line == 'B' || *line == 'C') {
@@ -205,7 +201,7 @@ int len;
 		new = (transtable *)malloc2(sizeof(transtable));
 		new -> org = strdup2(org);
 		new -> alias = strndup2(cp, l1);
-		new -> symlink = (l2 > 0) ? strndup2(eol, l2 - 1) : NULL;
+		new -> slink = (l2 > 0) ? strndup2(eol, l2 - 1) : NULL;
 		new -> type = *line;
 		new -> rdev = (maj >= (r_dev_t)0)
 			? makedev(maj, min) : (r_dev_t)-1;
@@ -235,7 +231,7 @@ transtable *tbl;
 			tp = tbl -> next;
 			free(tbl -> org);
 			free(tbl -> alias);
-			if (tbl -> symlink) free(tbl -> symlink);
+			if (tbl -> slink) free(tbl -> slink);
 			free(tbl);
 		}
 	}
@@ -433,8 +429,7 @@ struct stat *stp;
 {
 	transtable *tp;
 	char *cp;
-	u_short mode;
-	int i, n;
+	u_int mode;
 
 	if (!(cp = strrdelim(path, 0)) || !inittrans(path, cp++ - path))
 		return(-1);
@@ -446,11 +441,7 @@ struct stat *stp;
 	}
 	rr_curtbl = tp;
 
-	n = tolower2(tp -> type);
-	for (i = 0; typesymlist[i]; i++) if (n == typesymlist[i]) break;
-	if (!typesymlist[i]) mode = S_IFREG;
-	else mode = typelist[i];
-
+	if ((mode = getfmode(tp -> type)) == (u_int)-1) mode = S_IFREG;
 	stp -> st_mode &= ~S_IFMT;
 	stp -> st_mode |= mode;
 	if ((mode == S_IFBLK || mode == S_IFCHR) && tp -> rdev != (r_dev_t)-1)
@@ -477,9 +468,9 @@ int bufsiz;
 	}
 	rr_curtbl = tp;
 
-	if (tp -> type != 'L' || !(tp -> symlink)) return(-1);
-	len = strlen(tp -> symlink);
-	strncpy(buf, tp -> symlink, bufsiz);
+	if (tp -> type != 'L' || !(tp -> slink)) return(-1);
+	len = strlen(tp -> slink);
+	strncpy(buf, tp -> slink, bufsiz);
 	if (len > bufsiz) len = bufsiz;
 
 	return(len);
