@@ -4,6 +4,7 @@
  *	shell command module
  */
 
+#include <fcntl.h>
 #include "fd.h"
 #include "func.h"
 #include "kanji.h"
@@ -20,6 +21,16 @@ extern char *shortname __P_((char *, char *));
 #include "termemu.h"
 #endif
 
+#ifndef	O_TEXT
+#define	O_TEXT		0
+#endif
+
+typedef struct _localetable {
+	char *env;
+	char *val;
+	char *org;
+} localetable;
+
 extern int mark;
 extern off_t marksize;
 extern char fullpath[];
@@ -83,13 +94,24 @@ char *promptstr2 = NULL;
 #endif
 char *promptstr = NULL;
 char **history[2] = {NULL, NULL};
-char *histfile = NULL;
+char *histfile[2] = {NULL, NULL};
 short histsize[2] = {0, 0};
 short histno[2] = {0, 0};
-int savehist = 0;
+short savehist[2] = {0, 0};
 int n_args = 0;
 
 static short histbufsize[2] = {0, 0};
+static localetable localelist[] = {
+	{"LC_COLLATE", "C", NULL},
+	{"LC_CTYPE", "", NULL},
+	{"LC_MESSAGES", "C", NULL},
+	{"LC_MONETARY", "C", NULL},
+	{"LC_NUMERIC", "C", NULL},
+	{"LC_TIME", "C", NULL},
+	{"LC_ALL", NULL, NULL},
+	{"LANG", "C", NULL},
+};
+#define	LOCALELISTSIZ	arraysize(localelist)
 
 
 static int NEAR checksc(buf, ptr, arg)
@@ -200,11 +222,13 @@ int flags;
 	int len, optr;
 
 	if (!arg || !*arg) return(checksc(*bufp, ptr, NULL) - ptr);
-	if (!dir || !*dir) {
+	if (!dir) {
 #if	MSDOS && !defined (_NOUSELFN)
 		if ((flags & F_TOSFN) && shortname(arg, path) == path)
 			arg = path;
+		else
 #endif
+		arg = convput(conv, arg, 1, 0, NULL, NULL);
 	}
 	else {
 		strcatdelim2(path, dir, arg);
@@ -214,14 +238,13 @@ int flags;
 			if (iskanji1(arg, len)) len++;
 			if (arg[len] == _SC_) arg[len] = '/';
 		}
+#endif
+#ifndef	_NOKANJIFCONV
+		arg = kanjiconv2(conv, arg,
+			MAXPATHLEN - 1, DEFCODE, fnamekcode, L_FNAME);
 #endif
 	}
 
-	if (arg != path) arg = convput(conv, arg, 1, 0, NULL, NULL);
-#ifndef	_NOKANJIFCONV
-	else arg = kanjiconv2(conv, arg,
-		MAXPATHLEN - 1, DEFCODE, fnamekcode, L_FNAME);
-#endif
 	optr = ptr;
 	ptr = checksc(*bufp, ptr, arg);
 	arg = killmeta(arg);
@@ -266,7 +289,7 @@ int *ptrp;
 	if (!*ptrp) q = '\0';
 	pc = parsechar(&(s[*ptrp]), -1, '\0', 0, &q, NULL);
 	if (pc == PC_NORMAL) return(0);
-	if (pc == PC_WORD || pc == PC_META) {
+	if (pc == PC_WORD || pc == PC_ESCAPE) {
 		(*ptrp)++;
 		return(2);
 	}
@@ -660,6 +683,7 @@ macrostat *stp;
 				else if (c == 'C') code = CAP;
 				else if (c == 'U') code = UTF8;
 				else if (c == 'M') code = M_UTF8;
+				else if (c == 'I') code = I_UTF8;
 #  ifndef	_NOKANJIFCONV
 				else if (c == 'A') code = -1;
 #  endif
@@ -1262,7 +1286,7 @@ macrostat *stp;
 #endif	/* MACROMETA */
 
 	if (!(wastty = isttyiomode)) Xttyiomode(1);
-	cp = inputstr("", 0, p, command, HST_COM);
+	cp = inputstr(nullstr, 0, p, command, HST_COM);
 	if (!wastty) Xstdiomode();
 	free(command);
 	if (!cp) {
@@ -1292,7 +1316,7 @@ int flags;
 	char *cp, *tmp;
 	int n, ret;
 
-	while (*command == ' ' || *command == '\t') command++;
+	command = skipspace(command);
 	if ((cp = evalalias(command))) command = cp;
 
 	n = sigvecset(0);
@@ -1456,17 +1480,28 @@ int flags;
 	macrostat st;
 	FILE *fp;
 	char *tmp, *lang;
+	int i;
 
 	internal_status = FNC_FAIL;
 	st.flags = flags;
 	if (isinternalcomm(command)) st.flags |= F_ARGSET;
 
 	if (!(tmp = evalcommand(command, arg, &st))) return(NULL);
-	if ((lang = strdup2(getenv2("LANG")))) setenv2("LANG", "C", 1);
+	for (i = 0; i < LOCALELISTSIZ; i++) {
+		localelist[i].org = strdup2(getenv2(localelist[i].env));
+		if (!(localelist[i].val) || *(localelist[i].val)) {
+			setenv2(localelist[i].env, localelist[i].val, 1);
+			continue;
+		}
+		lang = getenv2("LC_ALL");
+		if (!lang || !*lang) lang = getenv2(localelist[i].env);
+		if (!lang || !*lang) lang = getenv2("LANG");
+		setenv2(localelist[i].env, lang, 1);
+	}
 	fp = popen2(tmp);
-	if (lang) {
-		setenv2("LANG", lang, 1);
-		free(lang);
+	for (i = 0; i < LOCALELISTSIZ; i++) {
+		setenv2(localelist[i].env, localelist[i].org, 1);
+		if (localelist[i].org) free(localelist[i].org);
 	}
 	free(tmp);
 
@@ -1566,14 +1601,14 @@ int uniq;
 
 	size = (int)histsize[n];
 	if (!history[n]) {
-		history[n] = (char **)malloc2(sizeof(char *) * (size + 1));
+		history[n] = (char **)malloc2((size + 1) * sizeof(char *));
 		for (i = 0; i <= size; i++) history[n][i] = NULL;
 		histbufsize[n] = size;
 		histno[n] = (short)0;
 	}
 	else if (size > histbufsize[n]) {
 		history[n] = (char **)realloc2(history[n],
-			sizeof(char *) * (size + 1));
+			(size + 1) * sizeof(char *));
 		for (i = histbufsize[n] + 1; i <= size; i++)
 			history[n][i] = NULL;
 		histbufsize[n] = size;
@@ -1621,37 +1656,34 @@ int n;
 	return(tmp);
 }
 
-int loadhistory(n, file)
+int loadhistory(n)
 int n;
-char *file;
 {
-	FILE *fp;
+	lockbuf_t *lck;
 	char *line;
 	int i, j, size;
 
-	if (!file || !(fp = Xfopen(file, "r"))) return(-1);
-#ifndef	NOFLOCK
-	if (lockfile(Xfileno(fp), LCK_READ) < 0) {
-		Xfclose(fp);
+	if (!histfile[n] || !histfile[n][0]) return(0);
+	lck = lockfopen(histfile[n], "r", O_TEXT | O_RDONLY);
+	if (!lck || !(lck -> fp)) {
+		lockclose(lck);
 		return(-1);
 	}
-#endif
 
 	size = (int)histsize[n];
-	history[n] = (char **)malloc2(sizeof(char *) * (size + 1));
+	history[n] = (char **)malloc2((size + 1) * sizeof(char *));
 	histbufsize[n] = size;
 	histno[n] = (short)0;
 
 	i = -1;
-	while ((line = fgets2(fp, 1))) {
+	while ((line = fgets2(lck -> fp, 1))) {
 		if (histno[n]++ >= MAXHISTNO) histno[n] = (short)0;
 		if (i < size) i++;
 		else free(history[n][i]);
 		for (j = i; j > 0; j--) history[n][j] = history[n][j - 1];
 		history[n][0] = line;
 	}
-	lockfile(Xfileno(fp), LCK_UNLOCK);
-	Xfclose(fp);
+	lockclose(lck);
 
 	for (i++; i <= size; i++) history[n][i] = NULL;
 
@@ -1675,36 +1707,35 @@ FILE *fp;
 	VOID_C Xfputc('\n', fp);
 }
 
-int savehistory(n, file)
+int savehistory(n)
 int n;
-char *file;
 {
-	FILE *fp;
+	lockbuf_t *lck;
 	int i, size;
 
+	if (!histfile[n] || !histfile[n][0] || savehist[n] <= 0) return(0);
 	if (!history[n] || !history[n][0]) return(-1);
-	if (!file || !(fp = Xfopen(file, "w"))) return(-1);
-#ifndef	NOFLOCK
-	if (lockfile(Xfileno(fp), LCK_WRITE) < 0) {
-		Xfclose(fp);
+	lck = lockfopen(histfile[n], "w",
+		O_TEXT | O_WRONLY | O_CREAT | O_TRUNC);
+	if (!lck || !(lck -> fp)) {
+		lockclose(lck);
 		return(-1);
 	}
-#endif
 
-	size = (savehist > (int)histsize[n]) ? (int)histsize[n] : savehist;
-	for (i = size - 1; i >= 0; i--) convhistory(history[n][i], fp);
-	lockfile(Xfileno(fp), LCK_UNLOCK);
-	Xfclose(fp);
+	size = (savehist[n] > histsize[n])
+		? (int)histsize[n] : (int)savehist[n];
+	for (i = size - 1; i >= 0; i--) convhistory(history[n][i], lck -> fp);
+	lockclose(lck);
 
 	return(0);
 }
 
-int parsehist(str, ptrp)
+int parsehist(str, ptrp, quote)
 char *str;
-int *ptrp;
+int *ptrp, quote;
 {
 	char *cp;
-	int i, n, ptr, size;
+	int i, n, pc, ptr, size;
 
 	ptr = (ptrp) ? *ptrp : 0;
 	size = (int)histsize[0];
@@ -1722,8 +1753,10 @@ int *ptrp;
 	}
 	else {
 		for (i = 0; str[ptr + i]; i++) {
-			if (strchr(CMDLINE_DELIM, str[ptr + i])) break;
-			if (iskanji1(str, ptr + i)) i++;
+			pc = parsechar(&(str[ptr + i]), -1,
+				'!', EA_FINDDELIM, &quote, NULL);
+			if (pc == PC_WORD || pc == PC_ESCAPE) i++;
+			else if (pc != PC_NORMAL && pc != PC_DQUOTE) break;
 		}
 		if (!i) n = ptr = -1;
 		else {
@@ -1738,7 +1771,10 @@ int *ptrp;
 
 	if (ptrp) *ptrp = ptr;
 	if (n < 0 || n >= size) return(-1);
-	if (!ptrp) {
+	if (ptrp) {
+		if (!history[0][n]) return(-1);
+	}
+	else {
 		while (!history[0][n]) if (--n < 0) return(-1);
 	}
 
@@ -1757,10 +1793,10 @@ char *command;
 	for (i = j = 0, quote = '\0'; command[i]; i++) {
 		cp = c_realloc(cp, j + 1, &size);
 		pc = parsechar(&(command[i]), -1, '!', 0, &quote, NULL);
-		if (pc == PC_WORD || pc == PC_META) cp[j++] = command[i++];
-		else if (pc == '!'){
+		if (pc == PC_WORD || pc == PC_ESCAPE) cp[j++] = command[i++];
+		else if (pc == '!') {
 			len = i++;
-			if ((n = parsehist(command, &i)) < 0) {
+			if ((n = parsehist(command, &i, quote)) < 0) {
 				if (i < 0) {
 					cp[j++] = '!';
 					i = len;
