@@ -380,10 +380,18 @@ extern int ignoreeof;
 extern int bgnotify;
 extern int jobok;
 #endif
-#if	defined (FD) && !defined (_NOEDITMODE)
+#ifdef	FD
+# ifndef	_NOEDITMODE
 extern int emacsmode;
 extern int vimode;
-#endif
+# endif
+# ifndef	_NOPTY
+extern int shptymode;
+# endif
+# if	!MSDOS
+extern int autosavetty;
+# endif
+#endif	/* FD */
 extern int loginshell;
 extern int noruncom;
 
