@@ -4,11 +4,11 @@
  *	terminal module
  */
 
+#include "machine.h"
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
 #include <fcntl.h>
-#include "machine.h"
 
 #if	defined (DJGPP) && (DJGPP >= 2)
 #include <time.h>
@@ -49,8 +49,8 @@ static int _asm_cli __P_((char *));
 #include <sys/time.h>
 
 typedef struct _kstree_t {
-	int key;
-	int num;
+	u_char key;
+	u_char num;
 	struct _kstree_t *next;
 } kstree_t;
 #endif	/* !MSDOS */
@@ -363,7 +363,7 @@ static char *NEAR tgetstr3 __P_((char **, char *, char *));
 static char *NEAR tgetkeyseq __P_((int, char *));
 static kstree_t *NEAR newkeyseqtree __P_((kstree_t *, int));
 static int NEAR freekeyseqtree __P_((kstree_t *, int));
-static int NEAR cmpkeyseq __P_((CONST VOID_P, CONST VOID_P));
+static int cmpkeyseq __P_((CONST VOID_P, CONST VOID_P));
 static int NEAR sortkeyseq __P_((VOID_A));
 static int putch3 __P_((tputs_t));
 #endif	/* !MSDOS */
@@ -446,10 +446,10 @@ static int specialkeycode[] = {
 	K_F(11), K_F(12), K_F(13), K_F(14), K_F(15),
 	K_F(16), K_F(17), K_F(18), K_F(19), K_F(20), K_HELP
 };
-#define	SPECIALKEYSIZ	((int)(sizeof(specialkeycode) / sizeof(int)))
+#define	SPECIALKEYSIZ	arraysize(specialkeycode)
 #else	/* !MSDOS */
 static char *dumblist[] = {"dumb", "un", "unknown"};
-#define	DUMBLISTSIZE	((int)sizeof(dumblist) / sizeof(char *))
+#define	DUMBLISTSIZE	arraysize(dumblist)
 static keyseq_t keyseq[K_MAX - K_MIN + 1];
 static kstree_t *keyseqtree = NULL;
 static char *defkeyseq[K_MAX - K_MIN + 1] = {
@@ -1282,7 +1282,7 @@ static int NEAR defaultterm(VOID_A)
 #if	!MSDOS
 	for (i = 0; i <= K_MAX - K_MIN; i++) keyseq[i].code = K_MIN + i;
 	for (i = 21; i <= 30; i++)
-		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | 01000;
+		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | K_ALTERNATE;
 #endif	/* !MSDOS */
 
 	return(0);
@@ -1354,7 +1354,7 @@ int *xp, *yp;
 	format = SIZEFMT;
 	keyflush();
 # if	MSDOS
-	for (i = 0; i < (int)sizeof(GETSIZE) - 1; i++)
+	for (i = 0; i < strsize(GETSIZE); i++)
 		bdos(0x06, GETSIZE[i], 0);
 # else
 	if (!usegetcursor) return(-1);
@@ -1362,41 +1362,42 @@ int *xp, *yp;
 	tflush();
 # endif
 
-	i = 0;
-	buf[i] = '\0';
+	buf[0] = '\0';
 	do {
-		if (!kbhit2(WAITKEYPAD * 1000L * 2)) break;
+		if (!kbhit2(WAITKEYPAD * 10000L)) break;
+# if	MSDOS
+		buf[0] = bdos(0x07, 0x00, 0);
+# else
+		if ((tmp = getch2()) == EOF) break;
+		buf[0] = tmp;
+# endif
+	} while (buf[0] != format[0]);
+
+	i = 0;
+	if (buf[0] == format[0]) for (i++; i < strsize(buf); i++) {
+		if (!kbhit2(WAITKEYPAD * 1000L)) break;
 # if	MSDOS
 		buf[i] = bdos(0x07, 0x00, 0);
 # else
 		if ((tmp = getch2()) == EOF) break;
 		buf[i] = tmp;
 # endif
-	} while (buf[i] != format[0]);
-
-	if (buf[i] == format[0]) while (i < (int)sizeof(buf) - 2) {
-		if (!kbhit2(WAITKEYPAD * 1000L)) break;
-# if	MSDOS
-		buf[++i] = bdos(0x07, 0x00, 0);
-# else
-		if ((tmp = getch2()) == EOF) break;
-		buf[++i] = tmp;
-# endif
-		if (buf[i] == format[sizeof(SIZEFMT) - 2]) break;
+		if (buf[i] == format[strsize(SIZEFMT) - 1]) break;
 	}
 	keyflush();
-	if (!i || buf[i] != format[sizeof(SIZEFMT) - 2]) return(-1);
-	buf[++i] = '\0';
+	while (kbhit2(WAITKEYPAD * 1000L)) VOID_C getch2();
+	if (!i || buf[i++] != format[strsize(SIZEFMT) - 1]) return(-1);
+	buf[i] = '\0';
 
 	count = 0;
 	val[0] = yp;
 	val[1] = xp;
 
 	for (i = j = 0; format[i] && buf[j]; i++) {
-		if (format[i] == '%' && format[++i] == 'd' && count < 2) {
-			tmp = getnum(buf, &j);
-			if (tmp < 0 || buf[j] != format[i + 1]) break;
-			*val[count++] = tmp;
+		if (format[i] == '%' && format[i + 1] == 'd') {
+			if ((tmp = getnum(buf, &j)) < 0) break;
+			i++;
+			if (count++ < 2) *(val[count - 1]) = tmp;
 		}
 		else if (format[i] != buf[j++]) break;
 	}
@@ -1666,20 +1667,20 @@ int num;
 	int i, n;
 
 	if (!parent || !(parent -> next))
-		new = (kstree_t *)malloc(sizeof(kstree_t) * num);
-	else new = (kstree_t *)realloc(parent -> next, sizeof(kstree_t) * num);
+		new = (kstree_t *)malloc(num * sizeof(kstree_t));
+	else new = (kstree_t *)realloc(parent -> next, num * sizeof(kstree_t));
 	if (!new) err2("realloc()");
 
 	if (!parent) n = 0;
 	else {
 		n = parent -> num;
-		parent -> num = num;
+		parent -> num = (u_char)num;
 		parent -> next = new;
 	}
 
 	for (i = n; i < num; i++) {
-		new[i].key = -1;
-		new[i].num = 0;
+		new[i].key = (u_char)-1;
+		new[i].num = (u_char)0;
 		new[i].next = (kstree_t *)NULL;
 	}
 
@@ -1693,13 +1694,14 @@ int n;
 	int i;
 
 	if (!list) return(-1);
-	for (i = list[n].num - 1; i >= 0; i--) freekeyseqtree(list[n].next, i);
+	for (i = (int)(list[n].num) - 1; i >= 0; i--)
+		freekeyseqtree(list[n].next, i);
 	if (!n) free(list);
 
 	return(0);
 }
 
-static int NEAR cmpkeyseq(vp1, vp2)
+static int cmpkeyseq(vp1, vp2)
 CONST VOID_P vp1;
 CONST VOID_P vp2;
 {
@@ -1736,7 +1738,7 @@ static int NEAR sortkeyseq(VOID_A)
 			}
 			if (k >= p -> num) {
 				newkeyseqtree(p, k + 1);
-				p -> next[k].key = i;
+				p -> next[k].key = (u_char)i;
 			}
 			p = &(p -> next[k]);
 		}
@@ -1774,7 +1776,7 @@ char *s;
 		if (i < DUMBLISTSIZE) dumbterm = 1;
 	}
 # ifdef	IRIX
-	winterm = !strncmp(term, WINTERMNAME, sizeof(WINTERMNAME) - 1);
+	winterm = !strncmp(term, WINTERMNAME, strsize(WINTERMNAME));
 # endif
 
 	for (;;) {
@@ -2061,7 +2063,7 @@ int setdefkeyseq(VOID_A)
 	}
 	for (i = 0; i <= K_MAX - K_MIN; i++) keyseq[i].code = K_MIN + i;
 	for (i = 21; i <= 30; i++)
-		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | 01000;
+		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | K_ALTERNATE;
 	sortkeyseq();
 
 	return(0);
@@ -2104,7 +2106,7 @@ int len;
 
 	if (str) {
 		for (i = 0; i <= K_MAX - K_MIN; i++) {
-			if ((keyseq[i].code & 0777) == n
+			if ((keyseq[i].code & ~K_ALTERNATE) == n
 			|| !(keyseq[i].str) || keyseq[i].len != len)
 				continue;
 			if (!memcmp(str, keyseq[i].str, len)) {
@@ -2153,7 +2155,8 @@ keyseq_t *list;
 				keyseq[i].str =
 					(char *)malloc(list[i].len + 1);
 				if (!keyseq[i].str) err2("malloc()");
-				memcpy(keyseq[i].str, list[i].str, list[i].len);
+				memcpy(keyseq[i].str, list[i].str,
+					list[i].len);
 			}
 		}
 		sortkeyseq();
@@ -2747,8 +2750,8 @@ u_char tbuf[];
 #endif
 
 /*ARGSUSED*/
-int getkey2(sig)
-int sig;
+int getkey2(sig, code)
+int sig, code;
 {
 #if	!defined (DJGPP) || defined (NOTUSEBIOS) || defined (PC98)
 	static u_char tbuf1[3] = {0xff, 0xff, 0xff};
@@ -2815,11 +2818,9 @@ int sig;
 int ungetch2(c)
 int c;
 {
-	if (ungetnum >= (int)sizeof(ungetbuf) / sizeof(u_char) - 1)
-		return(EOF);
-	if (ungetnum)
-		memmove(&(ungetbuf[1]), &(ungetbuf[0]),
-			ungetnum * sizeof(u_char));
+	if (ungetnum >= arraysize(ungetbuf)) return(EOF);
+	memmove((char *)&(ungetbuf[1]), (char *)&(ungetbuf[0]),
+		ungetnum * sizeof(u_char));
 	ungetbuf[0] = c;
 	ungetnum++;
 
@@ -2946,8 +2947,9 @@ int getch2(VOID_A)
 	return((int)ch);
 }
 
-int getkey2(sig)
-int sig;
+/*ARGSUSED*/
+int getkey2(sig, code)
+int sig, code;
 {
 	static int count = SENSEPERSEC;
 	kstree_t *p;
@@ -2973,8 +2975,12 @@ int sig;
 # if	!defined (_NOKANJICONV) || defined (CODEEUC)
 	else if (key != C_EKANA) /*EMPTY*/;
 #  if	!defined (_NOKANJICONV)
-	else if (inputkcode != EUC) /*EMPTY*/;
-#  endif
+#   ifdef	CODEEUC
+	else if (code != EUC && code != NOCNV) /*EMPTY*/;
+#   else
+	else if (code != EUC) /*EMPTY*/;
+#   endif
+#  endif	/* !_NOKANJICONV */
 	else {
 		if (!kbhit2(WAITMETA * 1000L) || (ch = getch2()) == EOF)
 			return(key);
@@ -3006,8 +3012,7 @@ int sig;
 			if (key == keyseq[p -> next[j].key].str[0]) break;
 		if (j >= p -> num) return(key);
 		p = &(p -> next[j]);
-		if (keyseq[p -> key].len == 1)
-			return(keyseq[p -> key].code);
+		if (keyseq[p -> key].len == 1) return(keyseq[p -> key].code);
 		if (!kbhit2(WAITKEYPAD * 1000L) || (ch = getch2()) == EOF)
 			return(key);
 	}
@@ -3031,18 +3036,18 @@ int sig;
 	return(key);
 }
 
-int getkey3(sig)
-int sig;
+int getkey3(sig, code)
+int sig, code;
 {
 	int ch;
 
-	ch = getkey2(sig);
+	if ((ch = getkey2(sig, code)) < 0) return(ch);
 	if (ch >= K_F('*') && ch < K_DL) {
 		if (ch == K_F('?')) ch = K_CR;
 		else ch -= K_F0;
 	}
 
-	return(ch & 0777);
+	return(ch & ~K_ALTERNATE);
 }
 
 int ungetch2(c)
@@ -3054,11 +3059,9 @@ int c;
 	ch = c;
 	Xioctl(ttyio, TIOCSTI, &ch);
 # else
-	if (ungetnum >= (int)sizeof(ungetbuf) / sizeof(u_char) - 1)
-		return(EOF);
-	if (ungetnum)
-		memmove(&(ungetbuf[1]), &(ungetbuf[0]),
-			ungetnum * sizeof(u_char));
+	if (ungetnum >= arraysize(ungetbuf)) return(EOF);
+	memmove((char *)&(ungetbuf[1]), (char *)&(ungetbuf[0]),
+		ungetnum * sizeof(u_char));
 	ungetbuf[0] = c;
 	ungetnum++;
 # endif
