@@ -4,11 +4,14 @@
  *	terminal I/O
  */
 
+#include "machine.h"
 #include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
-#include "machine.h"
-#include "termio.h"
+
+#ifdef	USESELECTH
+#include <sys/select.h>
+#endif
 
 #ifndef	NOUNISTDH
 #include <unistd.h>
@@ -18,6 +21,9 @@
 #include <stdlib.h>
 #endif
 
+#include "printf.h"
+#include "termio.h"
+
 #include <errno.h>
 #ifdef	NOERRNO
 extern int errno;
@@ -34,21 +40,21 @@ extern int errno;
 
 #if	defined (USESYSCONF) && defined (_SC_OPEN_MAX)
 #define	MAXOPENFILE	sysconf(_SC_OPEN_MAX)
-#else
+#else	/* !USESYSCONF || !_SC_OPEN_MAX */
 # ifdef	NOFILE
 # define	MAXOPENFILE	NOFILE
-# else
+# else	/* !NOFILE */
 #  ifdef	OPEN_MAX
 #  define	MAXOPENFILE	OPEN_MAX
-#  else
+#  else		/* !OPEN_MAX */
 #   if	MSDOS
 #   define	MAXOPENFILE	20
 #   else
 #   define	MAXOPENFILE	64
 #   endif
-#  endif
-# endif
-#endif
+#  endif	/* !OPEN_MAX */
+# endif	/* !NOFILE */
+#endif	/* !USESYSCONF || !_SC_OPEN_MAX */
 
 #ifndef	_PATH_TTY
 # if	MSDOS
@@ -74,6 +80,19 @@ extern int errno;
 #endif
 
 #define	K_CTRL(c)	((c) & 037)
+#define	MAXFDSET	256
+#define	MAXSELECT	16
+
+#ifndef	FD_SET
+typedef struct fd_set {
+	u_int fds_bits[1];
+} fd_set;
+# define	FD_SET(n, p)	(((p) -> fds_bits[0]) |= ((u_int)1 << (n)))
+# define	FD_ISSET(n, p)	(((p) -> fds_bits[0]) & ((u_int)1 << (n)))
+# define	FD_ZERO(p)	(((p) -> fds_bits[0]) = 0)
+# undef		MAXFDSET
+# define	MAXFDSET	(BITSPERBYTE * sizeof(u_int))
+#endif	/* !FD_SET */
 
 #if	!defined (FD) || defined (_NODOSDRIVE)
 #define	Xread		read
@@ -96,7 +115,7 @@ extern u_char _openfile[];
 static CONST u_short doserrlist[] = {
 	0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 15, 17, 18, 65, 80
 };
-#define	DOSERRLISTSIZ	((int)(sizeof(doserrlist) / sizeof(u_short)))
+#define	DOSERRLISTSIZ	arraysize(doserrlist)
 static CONST int unixerrlist[] = {
 	0, EINVAL, ENOENT, ENOENT, EMFILE, EACCES,
 	EBADF, ENOMEM, ENOMEM, ENOMEM, ENODEV, EXDEV, 0, EACCES, EEXIST
@@ -344,7 +363,7 @@ FILE **fpp;
 #endif
 
 	if (*fdp >= 0) fd = *fdp;
-	else if ((fd = newdup(open(_PATH_TTY, flags, 0600))) < 0) return(-1);
+	else if ((fd = newdup(open(_PATH_TTY, flags, 0666))) < 0) return(-1);
 	if (*fpp) fp = *fpp;
 	else if (!(fp = fdopen(fd, "w+b")) && !(fp = fopen(_PATH_TTY, "w+b")))
 		return(-1);
@@ -596,3 +615,44 @@ p_id_t Xfork(VOID_A)
 	return(pid);
 }
 #endif	/* CYGWIN */
+
+#if	!MSDOS \
+|| (!defined(NOTUSEBIOS) && defined (DJGPP) && (DJGPP >= 2) && !defined(PC98))
+int readselect(nfd, fds, result, vp)
+int nfd, fds[];
+char result[];
+VOID_P vp;
+{
+	fd_set readfds;
+	int i, n, max, dupfds[MAXSELECT];
+
+	FD_ZERO(&readfds);
+	max = -1;
+	if (nfd > MAXSELECT) nfd = MAXSELECT;
+
+	for (i = 0; i < nfd; i++) {
+		dupfds[i] = fds[i];
+		if (result) result[i] = 0;
+		if (fds[i] < 0) continue;
+		if (fds[i] >= MAXFDSET && (n = safe_dup(fds[i])) >= 0)
+			dupfds[i] = n;
+		if (dupfds[i] > max) max = dupfds[i];
+		FD_SET(dupfds[i], &readfds);
+	}
+	if (max++ < 0) return(0);
+
+	do {
+		n = select(max, &readfds, NULL, NULL, (struct timeval *)vp);
+	} while (n < 0 && errno == EINTR);
+	for (i = 0; i < nfd; i++)
+		if (dupfds[i] != fds[i]) safeclose(dupfds[i]);
+	if (n <= 0) return(n);
+
+	if (result) for (i = 0; i < nfd; i++) {
+		if (dupfds[i] < 0) continue;
+		if (FD_ISSET(dupfds[i], &readfds)) result[i] = 1;
+	}
+
+	return(n);
+}
+#endif	/* !MSDOS || (!NOTUSEBIOS && DJGPP && DJGPP >= 2 && !PC98) */
