@@ -146,6 +146,17 @@ typedef struct _strtable {
 	char *str;
 } strtable;
 
+typedef struct _lockbuf_t {
+	int fd;
+	FILE *fp;
+	char *name;
+	u_char flags;
+} lockbuf_t;
+
+#define	LCK_FLOCK		0001
+#define	LCK_INVALID		0002
+#define	LCK_STREAM		0004
+
 typedef struct _bindtable {
 	short key;
 	u_char f_func;
@@ -270,10 +281,13 @@ typedef struct _winvartable {
 
 extern winvartable winvar[];
 #ifdef	_NOSPLITWIN
-#define	win			0
 #define	windows			1
 #else
 extern int windows;
+#endif
+#if	defined (_NOSPLITWIN) && defined (_NOPTY)
+#define	win			0
+#else
 extern int win;
 #endif
 #ifndef	_NOARCHIVE
@@ -380,6 +394,28 @@ typedef struct _builtintable {
 #define	LCK_WRITE		1
 #define	LCK_UNLOCK		2
 
+#define	CHK_OK			0
+#define	CHK_EXIST		1
+#define	CHK_OVERWRITE		2
+#define	CHK_ERROR		(-1)
+#define	CHK_CANCEL		(-2)
+
+#define	APL_OK			0
+#define	APL_IGNORE		1
+#define	APL_ERROR		(-1)
+#define	APL_CANCEL		(-2)
+
+#define	CPP_UPDATE		1
+#define	CPP_RENAME		2
+#define	CPP_OVERWRITE		3
+#define	CPP_NOCOPY		4
+#define	CPP_FORWUPDATE		5
+#define	CPP_FORWOVERWRITE	6
+
+#define	RMP_BIAS		2
+#define	RMP_REMOVEALL		(RMP_BIAS + CHK_OK)
+#define	RMP_KEEPALL		(RMP_BIAS + CHK_ERROR)
+
 #define	TCH_MODE		00001
 #define	TCH_UID			00002
 #define	TCH_GID			00004
