@@ -54,11 +54,15 @@ char *path, *buf;
 	}
 	if (buf) {
 #ifndef	_NOUSELFN
-		if (shortname(path, buf) == buf);
+		if (shortname(path, buf) == buf) /*EMPTY*/;
 		else
 #endif
-		strcpy(buf, path);
+		unixrealpath(path, buf);
 	}
+#ifdef	DOUBLESLASH
+	if (isdslash(path)) drv = '_';
+	else
+#endif
 	drv = _dospath(path);
 
 	return((drv) ? drv : getcurdrv());
@@ -80,9 +84,9 @@ char *path;
 int dospath3(path)
 char *path;
 {
-	int drive;
+	int i, drive;
 
-	if ((drive = supportLFN(path)) >= 0 || drive <= -3) return(0);
+	if ((i = supportLFN(path)) >= 0 || i <= -3) return(0);
 
 	return((drive = _dospath(path)) ? drive : getcurdrv());
 }
@@ -93,6 +97,9 @@ char *path, *buf;
 	char *cp, tmp[MAXPATHLEN];
 	int i, drive;
 
+#ifdef	DOUBLESLASH
+	if (isdslash(path)) return(0);
+#endif
 	if ((drive = _dospath(path))) cp = path + 2;
 	else {
 		cp = path;
@@ -106,10 +113,7 @@ char *path, *buf;
 		strcpy(tmp, cp);
 		cp = tmp;
 	}
-	if (*cp == _SC_) {
-		*(buf++) = drive;
-		*(buf++) = ':';
-	}
+	if (*cp == _SC_) buf = gendospath(buf, drive, '\0');
 	else {
 		if (!dosgetcwd(buf, MAXPATHLEN)) return(0);
 		buf = strcatdelim(buf);
@@ -253,6 +257,7 @@ char *path;
 			else dd = lastdrv;
 		}
 		lastdrv = dd;
+		n = 0;
 	}
 #endif	/* !_NODOSDRIVE */
 	LOG1(_LOG_INFO_, n, "chdir(\"%k\");", path);
@@ -592,6 +597,18 @@ int whence;
 	return(ofs);
 }
 
+int Xftruncate(fd, len)
+int fd;
+off_t len;
+{
+	int n;
+
+	if (fd >= DOSFDOFFSET) n = dosftruncate(fd, len);
+	else n = ftruncate(fd, len);
+
+	return(n);
+}
+
 int Xdup(oldd)
 int oldd;
 {
